pro MSP_BDP_Parse_FITS_Header, Event
	;
	;	parse a fits header into the FITS structure
	;
	;	assumes header is loaded as a string array
	;	into FITS structure, also
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	;
	f_ptr = PTR_struct.PTR_FITS
	s_ptr = PTR_struct.PTR_summary
	;
	header = (*s_ptr).raw_header
	;
	;	grab the important stuff
	;
	(*f_ptr).image_id = 	FXPAR(header, 'OBSERNO')
	(*f_ptr).naxis1 = 		FXPAR(header, 'NAXIS1')
	(*f_ptr).naxis2 = 		FXPAR(header, 'NAXIS2')
	(*f_ptr).observer = 	FXPAR(header, 'OBSERVER')
	;
	test_CDS = FXPAR(header, 'CDS')
	type_CDS = SIZE(test_CDS, /TYPE)
	;
	if(type_CDS eq 7) then begin
		if(STRTRIM(test_CDS,2) eq "On") then (*f_ptr).CDS = 1 else (*f_ptr).CDS = 0
	endif else if(test_CDS) then (*f_ptr).CDS = 1 else (*f_ptr).CDS = 0
	;
	lst_string = FXPAR(header, 'LST-OBS')
	reads, lst_string, format='(i2,1x,i2,1x,i2)', hh, mm, ss
	(*f_ptr).LST_obs = double(hh) + double(mm)/60.0d0 + double(ss)/3600.0d0		; in decimal hours
	;
	ra_string = FXPAR(header, 'TELRA')
	reads, ra_string, format='(i2,1x,i2,1x,f4.1)', hh, mm, ss
	ra = double(hh) + double(mm) / 60.0d0 + ss / 3600.0d0
	ra = (!DPI / 180.0d0) * ra * 15.0d0
	(*f_ptr).ra_tele = ra										; in radians
	;
	dec_string = FXPAR(header, 'TELDEC')
	reads, dec_string, format='(i3,1x,i2,1x,i2)', hh, mm, ss
	hh_abs = abs(hh)
	dec = (!DPI / 180.0d0) * (double(hh_abs) + double(mm) / 60.0d0 + ss / 3600.0d0)
	if(hh lt 0) then dec = -1.0d0*dec
	(*f_ptr).dec_tele = dec										; in radians
	;
	(*f_ptr).airmass = FXPAR(header, 'AIRMASS')
	;
	;	is this a "new" header (for the 2010 season)
	;
	is_new_header = FXPAR(header, 'INSTVERS', COUNT=nh_count)
	if(nh_count gt 0) then begin
		if(STRUPCASE(STRTRIM(is_new_header,2)) eq 'V1.1') then header_version = 1 else header_version = 0
	endif else header_version=0
	;
	(*f_ptr).header_version = header_version
	;
	(*f_ptr).instrument = FXPAR(header, 'INSTRUME')
	;
	(*f_ptr).filter[0] = STRTRIM(FXPAR(header, 'FILTNME1'),2)
	(*f_ptr).filter[1] = STRTRIM(FXPAR(header, 'FILTNME2'),2)
	(*f_ptr).filter[2] = STRTRIM(FXPAR(header, 'FILTNME3'),2)
	(*f_ptr).filter[3] = STRTRIM(FXPAR(header, 'FILTNME4'),2)
	;
	;	if this is a "new" header, then need to cycle the filters by one from the 'old' header
	;
	if(header_version gt 0) then begin
		filt_save = (*f_ptr).filter[0]
		(*f_ptr).filter[0] = (*f_ptr).filter[1]
		(*f_ptr).filter[1] = (*f_ptr).filter[2]
		(*f_ptr).filter[2] = (*f_ptr).filter[3]
		(*f_ptr).filter[3] = filt_save
	endif
	;
	(*f_ptr).raw_filename = FXPAR(header, 'FILENAME')
	(*f_ptr).objname = 		FXPAR(header, 'OBJNAME')
	(*f_ptr).object = 		FXPAR(header, 'OBJECT')
	;
	(*f_ptr).date_obs = 	FXPAR(header, 'DATE-OBS')
	(*f_ptr).exptime = 		FXPAR(header, 'EXPTIME')
	;
	ut_string = 			FXPAR(header, 'UTCSTART')
;	reads, ut_string, format='(i2,1x,i2,1x,f5.2)', hh, mm, ss
;
;	replacement code as per Brian Taylor (coded by DPC 20101220)
;
	hms = STRSPLIT(ut_string, ':', /REGEX, /EXTRACT)
	n_parts =N_ELEMENTS(hms)
	if(n_parts ne 3) then begin
		print, 'ERROR parsing UT_STRING = ',ut_string
		print, 'Stopping'
		stop
	endif
	reads, STRTRIM(hms[0],2), hh, format='(i)'
	reads, STRTRIM(hms[1],2), mm, format='(i)'
	reads, STRTRIM(hms[2],2), ss, format='(f)'
;
;	end of replacement code
;
	ut = double(hh) + double(mm) / 60.0d0 + ss / 3600.0d0
	(*f_ptr).ut = ut											; in decimal hours
	;
	comments = 				FXPAR(header, 'COMMENT')
	nlines = 				N_ELEMENTS(comments)
	nmaxlines = 			N_ELEMENTS( (*f_ptr).comment )
	;
	;	patch 20080107 - writefits adds its own comment lines if stuff wraps longer
	;					than a normal line. This message has to be trapped and killed
	;					before it can contaminate the real COMMENTs we want
	;
	bad_line_1 = "This FITS file"
	bad_line_2 = "continued over"
	bad_line_3 = "character at"
	bad_line_4 = "on subsequent"
	;
	good_lines = 0L
	if(nlines ne 0) then begin
		for iline = 0, nlines-1 do begin
			test_1 = STRPOS( comments[iline], bad_line_1) eq -1
			test_2 = STRPOS( comments[iline], bad_line_2) eq -1
			test_3 = STRPOS( comments[iline], bad_line_3) eq -1
			test_4 = STRPOS( comments[iline], bad_line_4) eq -1
			if(test_1 and test_2 and test_3 and test_4) then begin
				(*f_ptr).comment[good_lines] = comments[iline]
				good_lines++
			endif
		endfor
	endif
	if(good_lines lt nmaxlines) then (*f_ptr).comment[good_lines:nmaxlines-1] = ''
	;
	;	if coadded, look for additional keywords
	;
    coaddTF = 			FXPAR(header,"COADDING")
    if(coaddTF) then begin
        numcoadds = 	FXPAR(header,"NUMCOADD")
        (*f_ptr).numcoadds = numcoadds
    endif else (*f_ptr).numcoadds = 1							; ie, no coadding
    ;
    ;	new (2010) fits values
    ;
    if(header_version eq 1) then begin
	    (*f_ptr).camera_pos = 	FXPAR(header, "CAMERA")
	    (*f_ptr).slit_pos = 	FXPAR(header, "S_DETENT")
	    (*f_ptr).decker_pos = 	FXPAR(header, "D_DETENT")
	    (*f_ptr).hwp_step = 	FXPAR(header, "HWP  ")
	    (*f_ptr).hwp_ang = 		normal_angle(FXPAR(header, "HWP_ANG"))
	    (*f_ptr).hwp_home = 	FXPAR(header, "H_HOME")
	endif else begin
		(*f_ptr).camera_pos = 	0
	    (*f_ptr).slit_pos = 	0
	    (*f_ptr).decker_pos = 	0
	    (*f_ptr).hwp_step = 	0
	    (*f_ptr).hwp_ang = 		0.0
	    (*f_ptr).hwp_home = 	0
	endelse
    ;
end
