;
;	Part 10 Event Handler Routines - Make Flat-Field Images
;
;-----------------------------------------------------------------
;
pro P10_Discover_Groups, Event, group_type
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	ff_ptr = PTR_struct.PTR_Flat
	;
	;	from the list of possible nights with flat data, build the list
	;	of groups of flat data
	;
	MSP_BDP_print_text, Event, "Discover Flat Groups Requested"
	;
	;	how many nights?
	;
	n_nights = (*ff_ptr).n_nights
	if(n_nights le 0) then begin
		MSP_BDP_print_text, Event, "ERROR - no nights with Flat Groups. Skipping."
		goto, P10DDG_end
	endif
	;
	;	for every night
	;
	n_flat_groups = 0L			; counter
	n_polar_flats = 0L
	;
	for inight = 0, n_nights-1 do begin
		;
		n_polar_images = 0L			; counter for polar flats
		;
		;	fetch night's desig
		;
		this_night_desig = (*ff_ptr).night_list[inight]
		this_night_desig_text = STRING(format='(I8.8)',this_night_desig)
		;
		;	load this night's SAV file
		;
		MSP_BDP_print_text, Event, "Opening Night Summary for night = "+this_night_desig_text
		;
		MSP_BDP_Create_Night_File_Path, Event, this_night_desig, night_file_path
		MSP_BDP_Restore_Night_File, Event, night_file_path, description, ierr
		;
		if(ierr ne 0) then begin
			MSP_BDP_print_text, Event, "Could not restore night "+this_night_desig_text+" - skipping"
			goto, night_loop_end
		endif
		;
		;	must redo pointers
		;
		WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
		s_ptr = PTR_struct.PTR_summary
		n_ptr = PTR_struct.PTR_night
		;
		;	capture number of groups
		;
		ngroups = (*n_ptr).num_groups
		ngroups_text = STRTRIM(STRING(format='(I3.1)',ngroups),2)
		MSP_BDP_print_text, Event, "Restored "+this_night_desig_text+" with "+ngroups_text+" groups"
		;
		;------------------------------------------------------------------------------------------
		;	for each group whose type matches group_type
		;
		if(ngroups le 0) then begin
			MSP_BDP_print_text, Event, "ERROR - no groups for this night. Skipping."
			goto, night_loop_end
		endif
		;
		;	set the directory path
		;
		dir_path = fix_path( (*s_ptr).root_reduced + '8_Linearity_Corrected_Images' + PATH_SEP() + $
					this_night_desig_text + PATH_SEP() )
		;
		for ig = 0, ngroups-1 do begin
			;
			;	get group type and compare to group_type
			;
			this_group_type = (*n_ptr).groups[ig].group_type
			tgt_UC = STRUPCASE(this_group_type)
			;
			test_1 = tgt_UC eq 'I-FLAT-ON'
			test_2 = tgt_UC eq 'I-FLAT-OFF'
			;
			test_3 = tgt_UC eq 'S-FLAT-ON'
			test_4 = tgt_UC eq 'S-FLAT-OFF'
			;
			test_5 = tgt_UC eq 'P-FLAT-ON'
			test_6 = tgt_UC eq 'P-FLAT-OFF'
			;
			mode_photometric = group_type eq 'phot'
			mode_polarimetric = group_type eq 'pol'
			;
			test_on = (test_1 or test_3 or test_5)
			test_off = (test_2 or test_4 or test_6)
			;
			final_test = 0
			;
			if(mode_photometric) then begin
				if(test_1 or test_2 or test_3 or test_4) then final_test = 1 else final_test = 0
			endif
			;
			if(mode_polarimetric) then begin
				if(test_5 or test_6) then final_test = 1 else final_test = 0
			endif
			;
			;	and make sure it is not subframed (for now)
			;
			frame_test = STRUPCASE((*n_ptr).groups[ig].group_frame) eq 'FULL'
			;
			if(final_test and frame_test) then begin
				;
				if(NOT(test_on) and NOT(test_off)) then begin
					MSP_BDP_print_text, Event, "ERROR - Coadd group "+STRING(format='(i4.1)',ig)+" is neither lights on nor lights off"
					goto, group_end
				endif
				;
				;	save crucial information
				;
				(*ff_ptr).night_desig[n_flat_groups] = this_night_desig
				;
				;	for photometric and spectroscopic flats...
				;
				if(mode_photometric) then begin
					(*ff_ptr).night_group_id[n_flat_groups] = (*n_ptr).groups[ig].group_id
					(*ff_ptr).exptime[n_flat_groups] = (*n_ptr).groups[ig].group_exptime
					(*ff_ptr).group_type[n_flat_groups] = this_group_type
					(*ff_ptr).group_filter[n_flat_groups] = STRTRIM((*n_ptr).groups[ig].group_filter,2)
					(*ff_ptr).group_camera[n_flat_groups] = STRTRIM((*n_ptr).groups[ig].group_camera,2)
					;
					;	HWP angle for pol images... TBD
					;
					(*ff_ptr).HWP_angle[n_flat_groups] = 0.0
					;
					;
					;	for each image in this group, build the full pathname to the _LC image
					;
					n_images = (*n_ptr).groups[ig].group_images
					(*ff_ptr).n_input_pathnames[n_flat_groups] = n_images
					;
					for i_image = 0, n_images-1 do begin
						;
						;	get the image number from the night's group list
						;
						image_no = (*n_ptr).groups[ig].group_list[i_image]
						if(i_image eq 0) then image_no_0 = image_no
						;
						;	build name of image - though we don't know if it exists...
						;
						filename = this_night_desig_text + '.' + $
								   STRTRIM(STRING(format='(i5.3)',image_no),2) + '_LC.fits'
						;
						;	full pathname
						;
						pathname = fix_path( FILEPATH(filename, ROOT_DIR = dir_path) )
						;
						(*ff_ptr).input_pathname[n_flat_groups,i_image] = pathname
						;
						;print, "input file, dir, path = ",filename, dir_path, pathname
						;
					endfor
					;
					;	build the JD for the first image
					;
					n_header_night = (*n_ptr).num_images
					ind = WHERE( (*n_ptr).image_list[0:n_header_night-1] eq image_no_0,count)
					if(count ne 1) then begin
						MSP_BDP_print_text, Event, "ERROR - did not match image no to night list"
						goto, P10DDG_end
					endif
					;
					this_image_UT = (*n_ptr).header_list[ind[0]].ut
					;
					year = this_night_desig / 10000L
					month = (this_night_desig - 10000L * year) / 100L
					day = this_night_desig - 10000L * year - 100L * month
					;
					hour = FLOOR(this_image_UT)
					minute = FLOOR((this_image_UT - double(hour))*60.0d0)
					second = 3600.0d0 * (this_image_UT - double(hour) - double(minute)/60.0d0)
					;
					this_jd = JULDAY(month, day, year, hour, minute, second)
					;
					(*ff_ptr).input_first_JD[n_flat_groups] = this_jd
					;
					;	build the output pathname for the group
					;
					if(test_on) then on_off_text = '_On_' else on_off_text = '_Off_'
					;
					out_file = 'Flt_' + (*ff_ptr).group_filter[n_flat_groups] + on_off_text + $
					STRING(format='(I6.6)',1000.0*(*ff_ptr).exptime[n_flat_groups]) + $
					'_' + (*ff_ptr).group_camera[n_flat_groups] + '_' + this_night_desig_text + '_g' + STRTRIM(STRING(format='(i4.1)',$
					(*ff_ptr).night_group_id[n_flat_groups]),2) + '.fits'
					;
					out_dir = (*s_ptr).root_reduced + '10_Flat_Field_Images' + PATH_SEP() + 'coadded_images' + PATH_SEP()
					;
					outpath = fix_path( FILEPATH(out_file, ROOT_DIR= out_dir) )
					(*ff_ptr).output_pathname[n_flat_groups] = outpath
					;
					;print, "output file, dir, path = ",out_file, out_dir, outpath
					;
					;	bump counter
					;
					n_flat_groups++
					;
				endif else begin
					;
					;	polarimetric flats - collect ALL the information first, then sort and digest it
					;
					n_images = (*n_ptr).groups[ig].group_images
					;
					for i_image = 0, n_images-1 do begin
						;
						;	get the image number from the night's group list
						;
						image_no = (*n_ptr).groups[ig].group_list[i_image]
						;
						n_header_night = (*n_ptr).num_images
						ind = WHERE( (*n_ptr).image_list[0:n_header_night-1] eq image_no,count)
						if(count ne 1) then begin
							MSP_BDP_print_text, Event, "ERROR - did not match image no to night list"
							goto, P10DDG_end
						endif
						;
						;	now can access the fits header info for this image
						;
						(*ff_ptr).polar_save.group[n_polar_images] = (*n_ptr).groups[ig].group_id
						(*ff_ptr).polar_save.image_no[n_polar_images] = image_no
						(*ff_ptr).polar_save.HWP[n_polar_images] = normal_angle((*n_ptr).image_HWP_angle[ind[0]])
						(*ff_ptr).polar_save.camera[n_polar_images] = (*n_ptr).image_camera[ind[0]]
						if(test_on) then (*ff_ptr).polar_save.lites[n_polar_images] = 'On'
						if(test_off) then (*ff_ptr).polar_save.lites[n_polar_images] = 'Off'
						(*ff_ptr).polar_save.filter[n_polar_images] = (*n_ptr).image_filter[ind[0]]
						;
						;	build name of image
						;
						filename = this_night_desig_text + '.' + $
								   STRTRIM(STRING(format='(i5.3)',image_no),2) + '_LC.fits'
						;
						;	full pathname
						;
						pathname = fix_path( FILEPATH(filename, ROOT_DIR = dir_path) )
						;
						(*ff_ptr).polar_save.pathname[n_polar_images] = pathname
						(*ff_ptr).polar_save.JD[n_polar_images] = get_image_JD(Event, pathname)
						;
						;print, "input file, dir, path = ",filename, dir_path, pathname
						;
						n_polar_images++
						(*ff_ptr).polar_save.n_images = n_polar_images
					endfor

					;
					;	for each HWP angle and each light condition, build the new "groups"
					;
;
;
;								out_file = 'Flt_' + (*ff_ptr).group_filter[n_flat_groups] + on_off_text + $
;								STRING(format='(I6.6)',1000.0*(*ff_ptr).exptime[n_flat_groups]) + $
;								'_' + STRTRIM(STRING(format='(I5.5)',100.0*(*ff_ptr).HWP_angle[n_flat_groups]),2) + $
;								'_' + (*ff_ptr).group_camera[n_flat_groups] + '_' + this_night_desig_text + '_g' + STRTRIM(STRING(format='(i4.1)',$
;								(*ff_ptr).night_group_id[n_flat_groups]),2) + '.fits'
;								;
;								out_dir = (*s_ptr).root_reduced + '10_Flat_Field_Images' + PATH_SEP() + 'polar_coadded_images' + PATH_SEP()
				endelse
			endif else begin
				;
				;print," Rejected group of type = ",this_group_type
				;
			endelse
			;
			group_end:
		endfor
		;
		night_loop_end:
		;
		;
		if(group_type eq 'pol' and n_polar_images ne 0) then begin
			;
			;	all images read in
			;
			MSP_BDP_print_text, Event, "Discovered a total of "+STRING(format='(I)',n_polar_images)+" polar flat images"
			;
			;	now, sort by HWP to build unique set of HWP groups (one each for On and Off)
			;
			HWP_inds = SORT( (*ff_ptr).polar_save.HWP[0:n_polar_images-1] )
			HWP_list_sort = normal_angle((*ff_ptr).polar_save.HWP[HWP_inds])
			HWP_uniq_inds = UNIQ(HWP_list_sort)
			HWP_uniq_list = HWP_list_sort[HWP_uniq_inds]
			N_HWP = N_ELEMENTS(HWP_uniq_list)
			;
			MSP_BDP_print_text, Event, "Found "+STRING(format='(I)',N_HWP)+' unique HWP angles'
			;
			;	and how many cameras?
			;
			CAM_inds = SORT( (*ff_ptr).polar_save.camera[0:n_polar_images-1] )
			CAM_list_sort = (*ff_ptr).polar_save.camera[CAM_inds]
			CAM_uniq_inds = UNIQ(CAM_list_sort)
			CAM_uniq_list = CAM_list_sort[CAM_uniq_inds]
			N_CAM = N_ELEMENTS(CAM_Uniq_list)
			;
			MSP_BDP_print_text, Event, "Found "+STRING(format='(I)',N_CAM)+' unique camera settings'
			;
			;	and how many filters?
			;
			FILT_inds = SORT( (*ff_ptr).polar_save.filter[0:n_polar_images-1] )
			FILT_list_sort = (*ff_ptr).polar_save.filter[CAM_inds]
			FILT_uniq_inds = UNIQ(FILT_list_sort)
			FILT_uniq_list = FILT_list_sort[FILT_uniq_inds]
			N_FILT = N_ELEMENTS(FILT_Uniq_list)
			;
			MSP_BDP_print_text, Event, "Found "+STRING(format='(I)',N_FILT)+' unique filter settings'
			;
			;	for each camera setting
			;
			for icam = 0, N_CAM-1 do begin
				;
				this_camera = CAM_uniq_list[icam]
				;
				;	for each filter setting
				;
				for ifilt = 0, N_FILT-1 do begin
					;
					this_filter = FILT_uniq_list[ifilt]
					;
					;	create the sub-groups
					;
					n_pol_groups = 2 * N_HWP
					;
					(*ff_ptr).polar_flat[n_polar_flats].n_polar_groups = n_pol_groups
					;
					i_pol_group = 0
					;
					for i_HWP = 0, N_HWP-1 do begin
						for i_lites = 0, 1 do begin
							;
							;	match up groups
							;
							if(i_lites eq 0) then lite_status = 'On' else lite_status = 'Off'
							;
							ind_match = WHERE( normal_angle((*ff_ptr).polar_save.HWP[0:n_polar_images-1]) eq HWP_uniq_list[i_HWP] $
							                   and lite_status eq (*ff_ptr).polar_save.lites[0:n_polar_images-1] $
							                   and this_camera eq (*ff_ptr).polar_save.camera[0:n_polar_images-1] $
							                   and this_filter eq (*ff_ptr).polar_save.filter[0:n_polar_images-1],count)
							;
							;	if count = 0, then there is a problem
							;
							if(count eq 0) then begin
								MSP_BDP_print_text, Event, "ERROR - no match to pol data for (filter, camera, lites, HWP)="+$
										STRING(this_filter, this_camera, lite_status, HWP_uniq_list[i_HWP],format='(a,1x,a,1x,a,1x,f6.2)')
								MSP_BDP_print_text, Event, "   no pathnames saved"
								(*ff_ptr).polar_flat[n_polar_flats].polar_group[i_pol_group].pathnames[0:0] = 'Null'
							endif else begin
								(*ff_ptr).polar_flat[n_polar_flats].polar_group[i_pol_group].pathnames[0:count-1] = (*ff_ptr).polar_save.pathname[ind_match]
							endelse
							(*ff_ptr).polar_flat[n_polar_flats].polar_group[i_pol_group].n_pathnames = count
							;
							;
							;print, "Pathnames for group ",i_pol_group," with HWP = ",HWP_uniq_list[i_HWP],", lites = ",lite_status,", camera = ",this_camera,", and ",count," images:"
							;for ic = 0, count-1 do begin
							;	print, (*ff_ptr).polar_flat[n_polar_flats].polar_group[i_pol_group].pathnames[ic]
							;endfor
							;
							;	save the other things we need
							;
							(*ff_ptr).polar_flat[n_polar_flats].polar_group[i_pol_group].lites = lite_status
							(*ff_ptr).polar_flat[n_polar_flats].polar_group[i_pol_group].night_desig = this_night_desig
							(*ff_ptr).polar_flat[n_polar_flats].polar_group[i_pol_group].camera = this_camera
							(*ff_ptr).polar_flat[n_polar_flats].polar_group[i_pol_group].filter = this_filter
							if(count ne 0) then begin
								(*ff_ptr).polar_flat[n_polar_flats].polar_group[i_pol_group].JD = (*ff_ptr).polar_save.JD[ind_match[0]]	; first image JD
							endif else begin
								(*ff_ptr).polar_flat[n_polar_flats].polar_group[i_pol_group].JD = 0.0
							endelse
							(*ff_ptr).polar_flat[n_polar_flats].polar_group[i_pol_group].HWP = HWP_uniq_list[i_HWP]
							;
							;	build the output pathname for the coadded image
							;
							filename = 'Pol_Flt_Coadd_'+STRING(format='(I8)',this_night_desig)+'_'+STRTRIM(this_camera)+$
										'_'+STRTRIM(this_filter)+$
							            '_Lites_'+STRTRIM(lite_status)+'_'+STRING(format='(I4.4)',FIX(10.0*HWP_uniq_list[i_HWP]))+$
							            '.fits'
							dir = (*s_ptr).root_reduced + '10_Flat_Field_Images' + PATH_SEP() + 'polar_coadded_images' + PATH_SEP()
							test = FILE_INFO(dir)
							if(NOT(test.exists) or NOT(test.directory)) then FILE_MKDIR, dir
							(*ff_ptr).polar_flat[n_polar_flats].polar_group[i_pol_group].output_pathname = 	FILEPATH(filename, ROOT_DIR = dir)
							;
							;print, "Ready to create pathname = ", (*ff_ptr).polar_flat[n_polar_flats].polar_group[i_pol_group].output_pathname
							;
							i_pol_group++
						endfor
					endfor
					;
					;	done with this filter's polar files, bump polar flat counter
					;
					n_polar_flats++
				endfor
				;
				;	done with this camera's polar files, save polar flat counter
				;
				(*ff_ptr).n_polar_flats = n_polar_flats
			endfor
			;
			;	and done with night
			;
			(*ff_ptr).n_nights++
		endif
		;
		;	any phot stuff??
		;
	endfor
	;
	;	update dark group counter, first with photometric flat group count
	;
	(*ff_ptr).n_flat_groups = n_flat_groups
	;
	;	then pol group count
	;
	(*ff_ptr).n_flat_groups = (*ff_ptr).n_flat_groups + (*ff_ptr).n_polar_flats
	;
	;	if no dark groups, say so and quit
	;
	if((*ff_ptr).n_flat_groups eq 0) then begin
		MSP_BDP_print_text, Event, "ERROR - No Flat Groups found. Skipping."
	endif else begin
		;
		;	turn on next button
		;
		if(mode_photometric) then P10_button_sensitize, Event, 1, 1
		if(mode_polarimetric) then P10_button_sensitize, Event, 10, 1
		;
	endelse
	;
	P10DDG_end:
end
;
;------------------------------------------------------
pro P10_Edit_Flat_Groups, Event, group_type
	;
	;	show the table of groups lined up for flat-field image creation
	;
	MSP_BDP_print_text, Event, "Show/Edit Groups for Flat-Field Image Computation requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	s_ptr = PTR_struct.PTR_summary
	n_ptr = PTR_struct.PTR_night
	r_ptr = PTR_struct.PTR_RAW1
	ff_ptr = PTR_struct.PTR_Flat
	;
	view_flag = (*ff_ptr).view_flag
	;
	;print, "group_type = ",group_type
	;
	if(group_type eq 'phot') then begin
		n_flat_groups = (*ff_ptr).n_flat_groups
	endif else begin
		n_polar_flats = (*ff_ptr).n_polar_flats
		n_polar_groups = lonarr(n_polar_flats)
		n_polar_groups = REFORM((*ff_ptr).polar_flat[0:n_polar_flats-1].n_polar_groups)
		i_polar_groups = lonarr(n_polar_flats)
		for i = 0, n_polar_flats-1 do begin
			i_polar_groups[i] = FIX(TOTAL(n_polar_groups[0:i]))		; total up to current group
		endfor
		;
		n_flat_groups = FIX(TOTAL(n_polar_groups))
	endelse
	;
	;------------------------------------------------------------------------------------
	;
	;	build the table structure
	;
	if(group_type eq 'phot') then begin
		col_headers = ['__Night__','Group', 'Group Type', 'Filter', 'ExpTime (ms)', 'Camera', 'N_Images', '___1st Image___', $
					'___Last Image___','___Output File Name___']
		ncols = 10
	endif else begin
		col_headers = ['__Night__','Group', 'Lites', 'Filter', 'HWP Angle', 'Camera', 'N_Images', '___1st Image___', $
					'___Last Image___','___Output File Name___']
		ncols = 11
	endelse
	;
	n_lines = FIX(n_flat_groups)
	format = strarr(ncols,n_lines)
	format[0:ncols-1,*] = '(a)'
	;
	color_array = strarr(ncols,n_lines)
	;
	row_headers = strarr(n_lines)
	if(group_type eq 'phot') then begin
		data_line = {night_desig: '', group_id: '', group_type: '', filter: '', $
					exptime: '', camera: '', n_images: '',$
				 	first: '', last: '', output_filename:''}
	endif else begin
		data_line = {night_desig: '', group_id: '', lites: '', filter: '', HWP_angle: '', $
					camera: '', n_images: '',$
				 	first: '', last: '', output_filename:''}
	endelse
	;
	data_array = REPLICATE(data_line, n_lines)			; array of structures
	;
	for i = 0, n_lines-1 do begin
			;
			if(group_type eq 'phot') then begin
				row_headers[i] = 'Flt_'+STRTRIM(STRING(format='(i4.1)',i),2)
				;
				night_desig_text = STRING(format='(I8.8)',(*ff_ptr).night_desig[i])
				data_array[i].night_desig = night_desig_text
				;
				g_text = STRTRIM(STRING(format='(i4.1)',(*ff_ptr).night_group_id[i]),2)
				data_array[i].group_id    = 'Gr_'+g_text
				;
				data_array[i].group_type = (*ff_ptr).group_type[i]
				data_array[i].filter = (*ff_ptr).group_filter[i]
				;
				data_array[i].exptime = STRING(format='(I6.6)',1000.0*(*ff_ptr).exptime[i])
				;
				data_array[i].camera = (*ff_ptr).group_camera[i]
				;
				data_array[i].n_images = STRTRIM(STRING(format='(i3.1)',(*ff_ptr).n_input_pathnames[i]),2)
				;
				first = FILE_BASENAME((*ff_ptr).input_pathname[i,0])
				last = FILE_BASENAME((*ff_ptr).input_pathname[i,(*ff_ptr).n_input_pathnames[i]-1])
				;
				data_array[i].first = first
				data_array[i].last = last
				;
				data_array[i].output_filename = FILE_BASENAME((*ff_ptr).output_pathname[i])
				;
				case data_array[i].group_type of
					"Skip": color = 'red'
					"**BAD**":	color = "red"
					"I-Flat-On": color = "goldenrod light"
					"I-Flat-Off": color = "goldenrod dark"
					"S-Flat-On": color = "rosybrown light"
					"S-Flat-Off": color = "rosybrown dark"
					"P-Flat-On": color = "palegreen light"
					"P-Flat-Off": color = "palegreen dark"
					else: color = "white"
				endcase
				;
			endif else begin
				;
				;	pol stuff
				;
				row_headers[i] = 'PFlt_'+STRTRIM(STRING(format='(i4.1)',i),2)
				;
				;	decode counter to infer which pol flat group
				;
				ind_high = WHERE(i ge i_polar_groups[*],high_count)
				;
;				if(high_count ne 0) then begin
;					current_flat = ind_high[high_count]		; highest one it is higher than
;				endif else begin
;					current_flat = 0
;				endelse
;
				current_flat = high_count
				;
				;	which group within this flat?
				;
				current_group = i
				if(current_flat gt 0) then current_group = current_group - i_polar_groups[current_flat-1]
				;
				;	ok, get data from structure
				;
				night_desig_text = STRING(format='(I8.8)',(*ff_ptr).polar_flat[current_flat].polar_group[current_group].night_desig)
				data_array[i].night_desig = night_desig_text
				;
				g_text = STRTRIM(STRING(format='(i4.1)',current_group),2)
				data_array[i].group_id    = 'Gr_'+g_text
				;
				lites = (*ff_ptr).polar_flat[current_flat].polar_group[current_group].lites
				data_array[i].lites = lites
				data_array[i].filter = (*ff_ptr).polar_flat[current_flat].polar_group[current_group].filter
				data_array[i].camera = (*ff_ptr).polar_flat[current_flat].polar_group[current_group].camera
				;
				data_array[i].HWP_angle = STRTRIM(STRING(format='(F8.2)',(*ff_ptr).polar_flat[current_flat].polar_group[current_group].HWP),2)
				;
				n_images = (*ff_ptr).polar_flat[current_flat].polar_group[current_group].n_pathnames
				data_array[i].n_images = STRTRIM(STRING(format='(i3.1)',n_images),2)
				;
				first = FILE_BASENAME((*ff_ptr).polar_flat[current_flat].polar_group[current_group].pathnames[0])
				last = FILE_BASENAME((*ff_ptr).polar_flat[current_flat].polar_group[current_group].pathnames[n_images-1])
				;
				data_array[i].first = first
				data_array[i].last = last
				;
				data_array[i].output_filename = FILE_BASENAME((*ff_ptr).polar_flat[current_flat].polar_group[current_group].output_pathname)
				;
				case lites of
					"Skip":color = 'red'
					"**BAD**":	color = "red"
					"On": color = "palegreen light"
					"Off": color = "palegreen dark"
					else: color = "white"
				endcase
				;
			endelse
			;
			;
			color_array[*,i] = color
	endfor
	;
	r_string = "Flat-Field Group List"
    new_struct = Dialog_Big_Table(Title="Flat-Field Image Group Table"+$
    				"    to turn off, put 'Skip' in group type column",Label=r_string, $
					Group_Leader=Event.top, Col_Headers = col_headers, $
					Format=format, Row_headers = row_headers,$
					Data_array = data_array, Font='Helvetica*16', $
					color_array=color_array)
	;
	;	load new table data back into structure
	;
	nc = N_TAGS(new_struct)
	nr = N_ELEMENTS(new_struct)
	;
	;	note - permit change of group_type, or lites only
	;
	for ir = 0, nr-1 do begin
		if(group_type eq 'phot') then begin
			if ( STRUPCASE(new_struct[ir].group_type) eq 'SKIP') then new_struct[ir].group_type = 'Skip'
			(*ff_ptr).group_type[ir] = 	new_struct[ir].group_type
		endif
	endfor
	;
	;	set view flag
	;
	(*ff_ptr).view_flag = 1L
	;
	if(group_type eq 'phot') then P10_button_sensitize, Event, 2, 1
	if(group_type eq 'pol') then P10_button_sensitize, Event, 11, 1
	;
	P10_ST_end:
end
;
;---------------------------------------------------
pro P10_Coadd_Groups, Event, group_type
	;
	if(group_type eq 'phot') then P10_Coadd_Groups_Phot, Event
	if(group_type eq 'pol') then P10_Coadd_Groups_Pol, Event
	;
end
;
;--------------------------------------------------
pro P10_Coadd_Groups_Pol, Event
	;
	;	performs coadding of each pol group
	;
	MSP_BDP_print_text, Event, "Starting Pol Flat Group Image Coadditions"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	ff_ptr = PTR_struct.PTR_Flat
	n_ptr = PTR_struct.PTR_night
	b_ptr = PTR_struct.PTR_BPM
	a_ptr = PTR_struct.PTR_Ave_RAW1
	;
	;	number of groups to process
	;
	n_polar_flats = (*ff_ptr).n_polar_flats
	n_polar_groups = lonarr(n_polar_flats)
	n_polar_groups = REFORM((*ff_ptr).polar_flat[0:n_polar_flats-1].n_polar_groups)
	i_polar_groups = lonarr(n_polar_flats)
	for i = 0, n_polar_flats-1 do begin
		i_polar_groups[i] = TOTAL(n_polar_groups[0:i])		; total up to current group
	endfor
	;
	n_groups = TOTAL(n_polar_groups)
	n_good = n_groups						; for now - could allow edits, later
	;
	;	clear LED progress bar to yellow
	;
	n_LEDs = N_ELEMENTS( (*s_ptr).P10_LED_colors )
	LED_scale = float(n_good) / float(n_LEDs)
	if(LED_scale le 1.0) then LED_scale = 1.0
	(*s_ptr).P10_LED_colors[*] = 'yellow'
	if(n_good lt n_LEDs) then (*s_ptr).P10_LED_colors[n_good:n_LEDs-1] = 'black'
	WCT_LED_stack, (*s_ptr).P10_LED_wid[1], (*s_ptr).P10_LED_colors
	;
	;	start by fetching BPM directory list
	;
	MSP_BDP_Read_BPM_Headers, Event
	n_bpm = (*b_ptr).bpm_n_BPM
	if(n_bpm eq 0) then begin
		MSP_BDP_print_text, Event, "ERROR - No Bad Pixel Masks found - go back to Part 5"
		goto, PP10SLC_outer_end
	endif
	;
	;	for every polar flat config and every group within that
	;
	i_good = 0L
	;
	for ipf = 0, n_polar_flats-1 do begin
		n_groups = n_polar_groups[ipf]
		;
		for ig = 0, n_groups -1 do begin
			;
			;	get night, group, number of images for each group to turn into one coadded image
			;
;			this_group_type = (*ff_ptr).group_type[ig]
			;
			;	if **BAD** skip
			;
;			if(STRUPCASE(this_group_type) eq '**BAD**') then begin
;				MSP_BDP_print_text, Event, 'Skipping group '+STRING(format='(I4.1)',ig)
;				goto, group_loop_end
;			endif
			;
			this_group = ig
			this_night = (*ff_ptr).polar_flat[ipf].polar_group[ig].night_desig
			n_input_images = (*ff_ptr).polar_flat[ipf].polar_group[ig].n_pathnames
			;
			night_desig_text = STRING(format='(I8.8)',this_night)
			;
			this_LED = float(i_good) / LED_scale
			this_LED = 0 > (this_LED < (n_LEDs-1))
			(*s_ptr).P10_LED_colors[this_LED] = 'orange'
			WCT_LED_stack, (*s_ptr).P10_LED_wid[1], (*s_ptr).P10_LED_colors
	        ;
			;	next, fetch the appropriate bad pixel mask, based on night and JD
			;	proximity
			;
			this_jd = (*ff_ptr).polar_flat[ipf].polar_group[ig].JD
			;
			d_jd = abs(this_jd - (*b_ptr).bpm_JD[0:n_bpm-1])
			d_jd_min = MIN(d_jd)
			ind = WHERE(d_jd_min eq d_jd[*],count)
			if(count ne 1) then begin
				MSP_BDP_print_text, Event, "ERROR - bad JD match in P10"
				goto, PP10SLC_outer_end
			endif
			;
			BPM_path = (*b_ptr).bpm_image_name[ind[0]]
			;
			input_path = (*ff_ptr).polar_flat[ipf].polar_group[ig].pathnames[0:n_input_images-1]
			;
			out_path = (*ff_ptr).polar_flat[ipf].polar_group[ig].output_pathname
			;
			(*s_ptr).P10_LED_colors[this_LED] = 'blue'
			WCT_LED_stack, (*s_ptr).P10_LED_wid[1], (*s_ptr).P10_LED_colors
			;
			MSP_BDP_print_text, Event, "Starting Polar Flat Field Image Coaddition for "+FILE_BASENAME(out_path)
			;
			;	 check that directory exists. if not, create it
			;
			out_dir = FILE_DIRNAME(out_path)
			test = FILE_INFO(out_dir)
			if(NOT(test.exists) or NOT(test.directory)) then FILE_MKDIR, out_dir
			;
			if(this_night ge (*s_ptr).row513_date) then row513_flag = 0 else row513_flag = 1
			;
			coadd_median_with_bad_sub, Event, BPM_path, input_path, out_path, $
									row513_flag, (*s_ptr).P10_LED_wid[1], this_LED, $
									(*s_ptr).P10_LED_colors
			;
			(*s_ptr).P10_LED_colors[this_LED] = 'green'
			WCT_LED_stack, (*s_ptr).P10_LED_wid[1], (*s_ptr).P10_LED_colors
			;
			MSP_BDP_print_text, Event,"Wrote output image "+out_path
			i_good++
			;
			group_loop_end:
		endfor
	endfor
	;
	P10_Save_Night_Summary, Event, 'pol'
	;
	P10_button_sensitize, Event, 12, 1
	;
	PP10SLC_outer_end:
end
;
;-------------------------------------------------
pro P10_Coadd_Groups_Phot, Event
	;
	;	performs coadding of each phot or spec group
	;
	MSP_BDP_print_text, Event, "Starting Flat Group Image Coadditions"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	ff_ptr = PTR_struct.PTR_Flat
	n_ptr = PTR_struct.PTR_night
	b_ptr = PTR_struct.PTR_BPM
	a_ptr = PTR_struct.PTR_Ave_RAW1
	;
	;	number of groups to process
	;
	n_groups = (*ff_ptr).n_flat_groups
	n_good = 0L
	for ig = 0, n_groups-1 do begin
		if( STRUPCASE( (*ff_ptr).group_type[ig]) ne '**BAD**' and (*ff_ptr).group_type[ig] ne 'Skip') then n_good++
	endfor
	;
	;	clear LED progress bar to yellow
	;
	n_LEDs = N_ELEMENTS( (*s_ptr).P10_LED_colors )
	LED_scale = float(n_good) / float(n_LEDs)
	if(LED_scale le 1.0) then LED_scale = 1.0
	(*s_ptr).P10_LED_colors[*] = 'yellow'
	if(n_good lt n_LEDs) then (*s_ptr).P10_LED_colors[n_good:n_LEDs-1] = 'black'
	WCT_LED_stack, (*s_ptr).P10_LED_wid[0], (*s_ptr).P10_LED_colors
	;
	;	start by fetching BPM directory list
	;
	MSP_BDP_Read_BPM_Headers, Event
	;
	;	how many did we load? If none, issue message and leave
	;
	n_bpm = (*b_ptr).bpm_n_BPM
	if(n_bpm eq 0) then begin
		MSP_BDP_print_text, Event, "ERROR - No Bad Pixel Masks found - go back to Part 5"
		goto, P10SLC_outer_end
	endif
	;
	;	for every group
	;
	i_good = 0L
	for ig = 0, n_groups -1 do begin
		;
		;	get night, group, number of images for each group to turn into one coadded image
		;
		this_group_type = (*ff_ptr).group_type[ig]
		this_group_filter = (*ff_ptr).group_filter[ig]
		;
		;	and original group number for testing for spec vs phot
		;
		orig_group_no = (*ff_ptr).night_group_id[ig]
		orig_group_type = STRUPCASE((*n_ptr).groups[orig_group_no].group_type)
		if(orig_group_type eq 'I-FLAT-ON' or orig_group_type eq 'I-FLAT-OFF') then sub_type = 'phot' else sub_type = 'spec'
		;
		;	if **BAD** skip
		;
		if(STRUPCASE(this_group_type) eq '**BAD**' or this_group_type eq 'Skip') then begin
			MSP_BDP_print_text, Event, 'Skipping group '+STRING(format='(I4.1)',ig)
			goto, group_loop_end
		endif
		;
		this_group = (*ff_ptr).night_group_id[ig]
		this_night = (*ff_ptr).night_desig[ig]
		n_input_images = (*ff_ptr).n_input_pathnames[ig]
		;
		night_desig_text = STRING(format='(I8.8)',this_night)
		;
		this_LED = float(i_good) / LED_scale
		this_LED = 0 > (this_LED < (n_LEDs-1))
		(*s_ptr).P10_LED_colors[this_LED] = 'orange'
		WCT_LED_stack, (*s_ptr).P10_LED_wid[0], (*s_ptr).P10_LED_colors
        ;
		;	next, fetch the appropriate bad pixel mask, based on night and JD
		;	proximity
		;
		this_jd = (*ff_ptr).input_first_JD[ig]
		;
		d_jd = abs(this_jd - (*b_ptr).bpm_JD[0:n_bpm-1])
		d_jd_min = MIN(d_jd)
		ind = WHERE(d_jd_min eq d_jd[*],count)
		if(count ne 1) then begin
			MSP_BDP_print_text, Event, "ERROR - bad JD match in P10"
			goto, P10SLC_outer_end
		endif
		;
		BPM_path = (*b_ptr).bpm_image_name[ind[0]]
		;
		;MSP_BDP_print_text, Event, "  BPM file selected = "+FILE_BASENAME(BPM_path)
		;
		;	summarize input files
		;
		input_path = (*ff_ptr).input_pathname[ig,0:n_input_images-1]
		;
		out_path = (*ff_ptr).output_pathname[ig]
		;
		(*s_ptr).P10_LED_colors[this_LED] = 'blue'
		WCT_LED_stack, (*s_ptr).P10_LED_wid[0], (*s_ptr).P10_LED_colors
		;
		MSP_BDP_print_text, Event, "Starting Flat Field Image Coaddition for "+FILE_BASENAME(out_path)
		;
		;	 check that directory exists. if not, create it
		;
		out_dir = FILE_DIRNAME(out_path)
		test = FILE_INFO(out_dir)
		if(NOT(test.exists) or NOT(test.directory)) then FILE_MKDIR, out_dir
		;
		if(this_night ge (*s_ptr).row513_date) then row513_flag = 0 else row513_flag = 1
		;
		coadd_median_with_bad_sub, Event, BPM_path, input_path, out_path, $
								row513_flag, (*s_ptr).P10_LED_wid[0], this_LED, $
								(*s_ptr).P10_LED_colors
		;
		;	check that filter names are OK
		;
		;print, "checking filter names for filter-type = ",this_group_filter
		;
		Test_Fix_Bad_Filter_Names, Event, this_group_filter, PATHNAME=out_path
		;
		(*s_ptr).P10_LED_colors[this_LED] = 'green'
		WCT_LED_stack, (*s_ptr).P10_LED_wid[0], (*s_ptr).P10_LED_colors
		;
		MSP_BDP_print_text, Event,"Wrote output image "+out_path
		i_good++
		;
		group_loop_end:
	endfor
	;
	P10_Save_Night_Summary, Event, 'phot'
	;
	P10_button_sensitize, Event, 3, 1
	;
	P10SLC_outer_end:
end
;
;--------------------------------------------------------------------------------------
pro P10_Match_On_Off_Pairs, Event, group_type
	;
	;	read the coadd directory to build list of images, then match on-off pairs with
	;	otherwise identical properties.
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	a_ptr = PTR_struct.PTR_Ave_RAW1
	ff_ptr = PTR_struct.PTR_Flat
	;
	MSP_BDP_print_text, Event, "Match On-Off Image Pairs requested"
	;
	if(group_type eq 'phot') then coadd_dir = (*s_ptr).root_reduced + '10_Flat_Field_Images' + PATH_SEP() + 'coadded_images'
	if(group_type eq 'pol') then coadd_dir = (*s_ptr).root_reduced + '10_Flat_Field_Images' + PATH_SEP() + 'polar_coadded_images'
	;
	test = FILE_INFO(coadd_dir)
	if(NOT(test.exists) or NOT(test.directory)) then begin
		MSP_BDP_print_text, Event, "ERROR - coadd directory problems"
		goto, P10MOOP_end
	endif
	;
	;	load image list
	;
	image_list = FILE_SEARCH(coadd_dir+PATH_SEP() + '*.fits',count=n_images)
	if(n_images le 0) then begin
		MSP_BDP_print_text, Event, "ERROR - coadd directory empty"
		goto, P10MOOP_end
	endif
	;
	if(group_type eq 'phot') then f_struct = {pathname: '', filter: '', on_off: '', expt: '', camera: '', night_desig: '', $
				group: '', imatch: -1L}
	if(group_type eq 'pol') then f_struct = {pathname: '', filter: '', on_off: '', camera: '', night_desig: '', $
				group: '', HWP: '', imatch: -1L}
	;
	(*ff_ptr).fm_ptr = PTR_NEW(REPLICATE(f_struct,n_images))
	;
	;	deconstruct names into properties
	;
	for i = 0, n_images-1 do begin
		part = STRSPLIT(FILE_BASENAME(image_list[i]),'_',/REGEX,/EXTRACT,count=n_parts)
		;
		if(group_type eq 'phot') then begin
			(*(*ff_ptr).fm_ptr)[i].pathname = image_list[i]
			(*(*ff_ptr).fm_ptr)[i].filter = part[1]
			(*(*ff_ptr).fm_ptr)[i].on_off = part[2]
			(*(*ff_ptr).fm_ptr)[i].expt = part[3]
			(*(*ff_ptr).fm_ptr)[i].camera = part[4]
			(*(*ff_ptr).fm_ptr)[i].night_desig = part[5]
			(*(*ff_ptr).fm_ptr)[i].group = part[6]
			(*(*ff_ptr).fm_ptr)[i].imatch = -1L
			;
			print, "image ",image_list[i]
			print, "on-off=",part[2],' filter = ',part[1], ', camera=',part[4],', night=',part[5],$
							', group = ',part[6], ' expT = ',part[3]
			;
		endif else begin
			(*(*ff_ptr).fm_ptr)[i].camera = part[4]
			(*(*ff_ptr).fm_ptr)[i].night_desig = part[3]
			(*(*ff_ptr).fm_ptr)[i].group = STRING(format='(1x,I)',i)
			(*(*ff_ptr).fm_ptr)[i].imatch = -1L
			;
			;	if only 7 items, use old parsing - DPC - 20101227
			;
			if(n_parts eq 7) then begin
				(*(*ff_ptr).fm_ptr)[i].pathname = image_list[i]
				(*(*ff_ptr).fm_ptr)[i].filter = 'H-Pol'
				(*(*ff_ptr).fm_ptr)[i].on_off = part[6]
				;
				;print, "part[7] = ",part[7]
				sub_part = STRMID(part[7],0,4)
				;print, "sub_part = ",sub_part
				;
				(*(*ff_ptr).fm_ptr)[i].HWP = sub_part
			endif else begin
			;
			;	if 8 items, use new parsing
			;
				(*(*ff_ptr).fm_ptr)[i].pathname = image_list[i]
				(*(*ff_ptr).fm_ptr)[i].filter = part[5]
				(*(*ff_ptr).fm_ptr)[i].on_off = part[7]
				;
				;print, "part[7] = ",part[7]
				sub_part = STRMID(part[8],0,4)
				;print, "sub_part = ",sub_part
				;
				(*(*ff_ptr).fm_ptr)[i].HWP = sub_part
			endelse
			;
			;print, "image ",image_list[i]
			;print, "on-off=",part[6],', camera=',part[4],', night=',part[3],', HWP=',sub_part
			;
		endelse
		;
	endfor
	;
	;	match pairs
	;
	(*ff_ptr).match[*,0:n_images-1] = -1L 			; first = On, second = Off
	(*ff_ptr).n_match = 0L
	;
	for i = 0, n_images-1 do begin
		(*(*ff_ptr).fm_ptr)[i].imatch = -1L
	endfor
	;
	for i = 0, n_images-1 do begin
		;
		;	check imatch for this item
		;
		if( (*(*ff_ptr).fm_ptr)[i].imatch eq -1L) then begin
			;
			;	not yet matched. Look at all others for a match
			;
			this_night = (*(*ff_ptr).fm_ptr)[i].night_desig
			this_filter = (*(*ff_ptr).fm_ptr)[i].filter
			if(group_type eq 'phot') then this_expt = (*(*ff_ptr).fm_ptr)[i].expt
			if(group_type eq 'pol') then this_HWP = (*(*ff_ptr).fm_ptr)[i].HWP
			this_camera = (*(*ff_ptr).fm_ptr)[i].camera
			this_on_off = (*(*ff_ptr).fm_ptr)[i].on_off
			this_group = (*(*ff_ptr).fm_ptr)[i].group
			this_group_no = 0
			reads, this_group, format='(1x,i)', this_group_no
			;
			;	start all matches with the ON image
			;
			if(this_on_off eq 'Off') then goto, i_loop_end
			;
			for j = 0, n_images-1 do begin
				if( (*(*ff_ptr).fm_ptr)[j].imatch eq -1L and i ne j) then begin
					if(this_night ne (*(*ff_ptr).fm_ptr)[j].night_desig) then goto, j_loop_end
					if(this_filter ne (*(*ff_ptr).fm_ptr)[j].filter) then goto, j_loop_end
					if(group_type eq 'phot') then begin
						if(this_expt ne (*(*ff_ptr).fm_ptr)[j].expt) then goto, j_loop_end
					endif
					if(group_type eq 'pol') then begin
						if(this_HWP ne (*(*ff_ptr).fm_ptr)[j].HWP) then goto, j_loop_end
					endif
					if(this_camera ne (*(*ff_ptr).fm_ptr)[j].camera) then goto, j_loop_end
					;
					;	but opposite on-off case
					;
					if(this_on_off eq (*(*ff_ptr).fm_ptr)[j].on_off) then goto, j_loop_end
					;
					;	if phot check the group number - it should be within 4 of the one looking
					;			(upped to 4 from 2 - 20080803 DPC - to account for observing errors)
					;
					gap_range = 4
					if(this_night ge 20120000) then gap_range = 2
					;
					if(group_type eq 'phot') then begin
						trial_group = (*(*ff_ptr).fm_ptr)[j].group
						trial_group_no = 0
						reads, trial_group, format='(1x,i)', trial_group_no
						;
						if( abs(this_group_no - trial_group_no) gt gap_range) then goto, j_loop_end
					endif
					;
					;	ok, this is the pair - mark them using imatch
					;
					(*(*ff_ptr).fm_ptr)[i].imatch = j
					(*(*ff_ptr).fm_ptr)[j].imatch = i
					(*ff_ptr).match[0,(*ff_ptr).n_match] = i
					(*ff_ptr).match[1,(*ff_ptr).n_match] = j
					;
					;	create difference pathname while here
					;
					if(group_type eq 'phot') then (*ff_ptr).difference_pathname[(*ff_ptr).n_match] = (*s_ptr).root_reduced + '10_Flat_Field_Images' + $
								PATH_SEP() + 'difference_images' + PATH_SEP() + 'Dif_' + this_filter + '_' + $
								this_expt + '_' + this_camera + '_' + this_night + '_' + this_group + '.fits'
					if(group_type eq 'pol') then (*ff_ptr).difference_pathname[(*ff_ptr).n_match] = (*s_ptr).root_reduced + '10_Flat_Field_Images' + $
								PATH_SEP() + 'polar_difference_images' + PATH_SEP() + 'Dif_' + this_night + '_' + $
								this_filter + '_' + this_camera + '_HWP_' + this_HWP + '.fits'
					;
					(*ff_ptr).n_match++
					;
					;	and blow out of this loop to prevent multiple matches
					;
					goto, i_loop_end
				endif
				j_loop_end:
			endfor
			;
			;	if we didn't match this one, leave its imatch value at -1
			;
		endif
		i_loop_end:
	endfor
	;
	;	any non-matched?
	;
	(*ff_ptr).n_unmatch = 0L
	n_match = (*ff_ptr).n_match
	for i = 0, n_images-1 do begin
		if( (*(*ff_ptr).fm_ptr)[i].imatch eq -1L) then begin
			(*ff_ptr).match[0,n_match+(*ff_ptr).n_unmatch] = i
			(*ff_ptr).match[1,n_match+(*ff_ptr).n_unmatch] = -1L
			(*ff_ptr).n_unmatch++
		endif
	endfor
	;
	;	turn on next button
	;
	if(group_type eq 'phot') then P10_button_sensitize, Event, 4, 1
	if(group_type eq 'pol') then P10_button_sensitize, Event, 13, 1
	MSP_BDP_print_text, Event, "Matching found "+STRING(format='(I3.1)',(*ff_ptr).n_match)+' matches'
	if( (*ff_ptr).n_unmatch ne 0L) then begin
		MSP_BDP_print_text, Event, "  also found "+STRING(format='(I3.1)',(*ff_ptr).n_unmatch)+' non-matches'
	endif
	;
	;PTR_FREE, (*ff_ptr).fm_ptr
	;
	P10MOOP_end:
end
;
;--------------------------------------------------------------------------------------
pro P10_View_On_Off_Pairs, Event, group_type
	;
	;	list matches on same lines, also call out non-matches
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	a_ptr = PTR_struct.PTR_Ave_RAW1
	ff_ptr = PTR_struct.PTR_Flat
	;
	MSP_BDP_print_text, Event, "View Matches requested"
	;
	;	build the table structure
	;
	col_headers = ['_______Lights-On Filename_______', '_______Lights-Off Filename_______', $
				'_______Output Filename_______']
	ncols = 3
	;
	n_lines = (*ff_ptr).n_match + (*ff_ptr).n_unmatch
	format = strarr(ncols,n_lines)
	format[0:ncols-1,*] = '(a)'
	;
	color_array = strarr(ncols,n_lines)
	;
	row_headers = strarr(n_lines)
	data_line = {on_filename: '', off_filename: '', difference_filename:''}
	;
	data_array = REPLICATE(data_line, n_lines)			; array of structures
	;
	for i = 0, n_lines-1 do begin
			;
			row_headers[i] = 'Flt_'+STRTRIM(STRING(format='(i4.1)',i),2)
			;
			data_array[i].on_filename = FILE_BASENAME( (*(*ff_ptr).fm_ptr)[(*ff_ptr).match[0,i]].pathname )
			if( (*ff_ptr).match[1,i] eq -1L) then begin
				data_array[i].off_filename = 'none'
				data_array[i].difference_filename = 'no match found'
			endif else begin
				data_array[i].off_filename = FILE_BASENAME( (*(*ff_ptr).fm_ptr)[(*ff_ptr).match[1,i]].pathname )
				data_array[i].difference_filename = FILE_BASENAME( (*ff_ptr).difference_pathname[i] )
			endelse
			;
			color_array[*,i] = "white"
			;
			case STRUPCASE(data_array[i].off_filename) of
				"NONE":	color = "red"
				"**BAD**": color = "red"
				"Skip": color = 'red'
				else: color = "green"
			endcase
			;
			color_array[*,i] = color
	endfor
	;
	r_string = "Flat-Field Match List"
    new_struct = Dialog_Big_Table(Title="Flat-Field Match Table"+$
    				"   or turn off group by putting 'Skip' in Lights-Off column",Label=r_string, $
					Group_Leader=Event.top, Col_Headers = col_headers, $
					Format=format, Row_headers = row_headers,$
					Data_array = data_array, Font='Helvetica*16', $
					color_array=color_array)
	;
	;	turn off differencing of unwanted pairs by setting off to "none"
	;
	;	load new table data back into structure
	;
	nc = N_TAGS(new_struct)
	nr = N_ELEMENTS(new_struct)
	;
	;	note - permit change of group_type, only
	;
	for ir = 0, nr-1 do begin
		test_off = STRUPCASE(new_struct[ir].off_filename)
		if(test_off eq 'NONE' or test_off eq '**BAD**' or test_off eq 'SKIP') then begin
			(*ff_ptr).match[1,ir] = -1
		endif

	endfor
	;
	if(group_type eq 'phot') then P10_button_sensitize, Event, 5, 1
	if(group_type eq 'pol') then P10_button_sensitize, Event, 14, 1
	;
end
;
;--------------------------------------------------------------------------------------
pro P10_Compute_Difference_Images, Event, group_type
	;
	;	compute difference image, but respect bad pixels
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	a_ptr = PTR_struct.PTR_Ave_RAW1
	ff_ptr = PTR_struct.PTR_Flat
	;
	MSP_BDP_print_text, Event, "Difference Image Pairs requested"
	;
	;	 verify that output directory exits, if not, create it
	;
	if(group_type eq 'phot') then outdir = (*s_ptr).root_reduced + PATH_SEP() + '10_Flat_Field_Images' + PATH_SEP() + 'difference_images'
	if(group_type eq 'pol') then outdir = (*s_ptr).root_reduced + PATH_SEP() + '10_Flat_Field_Images' + PATH_SEP() + 'polar_difference_images'
	test = FILE_INFO(outdir)
	if(NOT(test.exists) or NOT(test.directory)) then begin
		FILE_MKDIR, outdir
	endif
	;
	bad_pixel_value = -1.0e6
	;
	n_pairs = (*ff_ptr).n_match		; ignore non-matches
	;
	for ip = 0, n_pairs-1 do begin
		;
		;	test for flag to skip this pair
		;
		if( (*ff_ptr).match[1,ip] eq -1) then goto, pair_loop_end
		;
		;	fetch on, off images
		;
		on_pathname = fix_path(	(*(*ff_ptr).fm_ptr)[(*ff_ptr).match[0,ip]].pathname )
		off_pathname = fix_path( (*(*ff_ptr).fm_ptr)[(*ff_ptr).match[1,ip]].pathname )
		;
		on_array = readfits(on_pathname, on_header, /silent)
		off_array = readfits(off_pathname, off_header, /silent)
		;
		array_size = SIZE(on_array)
		nc = array_size[1]
		nr = array_size[2]
		;
		diff_array = on_array * 0.0
		;
		for ic = 0, nc-1 do begin
			for ir = 0, nr-1 do begin
				on_pix = on_array[ic,ir]
				off_pix = off_array[ic,ir]
				if(on_pix eq bad_pixel_value or $
				   ~FINITE(on_pix) or $
				   off_pix eq bad_pixel_value or $
				   ~FINITE(off_pix) ) then begin
					diff_array[ic,ir] = bad_pixel_value
				endif else begin
					diff_array[ic,ir] = on_pix - off_pix
				endelse
			endfor
		endfor
		;
		;	update header and write to disk
		;
		FXADDPAR, on_header, 'FILENAME', FILE_BASENAME((*ff_ptr).difference_pathname[ip]), 'Difference Flat Image'
		FXADDPAR, on_header, 'DIRNAME', FILE_DIRNAME((*ff_ptr).difference_pathname[ip],/mark_directory), 'Difference Flat Directory'
		FXADDPAR, on_header, 'HISTORY', 'Formed Difference Image on '+SYSTIME()+' from:'
		FXADDPAR, on_header, 'HISTORY','   reduced by '+(*s_ptr).who
		FXADDPAR, on_header, 'HISTORY', '  Lights-On Image = '+FILE_BASENAME(on_pathname)
		FXADDPAR, on_header, 'HISTORY', '  Lights-Off Image= '+FILE_BASENAME(off_pathname)
		;
		;	test for this difference being spectoscopic
		;
;		on_group = (*(*ff_ptr).fm_ptr)[(*ff_ptr).match[0,ip]].group
;		group_sub_type = STRUPCASE((*n_ptr).groups[on_group].group_type)
;
;		if(group_sub_type eq 'S-FLAT-ON' or group_sub_type eq 'S-FLAT-OFF') then begin
;			;
;			;	is, do simple dif and write
;			;
;			writefits, (*ff_ptr).difference_pathname[ip], diff_array, on_header
;		endif else begin
			;
			;	nope - is phot or pol - flatten background and then write
			;
;			on_array = model_background( diff_array, SKY=sky)
;			;
;			FXADDPAR, on_header, 'HISTORY', '  Flattened background. Sky = '+STRING(sky)
;			MSP_BDP_print_text, Event, "Flattened Background. Sky = "+STRING(sky)
;			writefits, (*ff_ptr).difference_pathname[ip], on_array, on_header
			;
			;	back to no flattening - DPC - 20091217
			;
			writefits, fix_path( (*ff_ptr).difference_pathname[ip] ), diff_array, on_header
;		endelse
		;
		MSP_BDP_print_text, Event, "Wrote image "+FILE_BASENAME((*ff_ptr).difference_pathname[ip])+" to disk"
		;
		pair_loop_end:
	endfor
	;
	MSP_BDP_print_text, Event, "Created "+STRING(format='(I4.1)',n_pairs)+' Difference Images'
	;
	if(group_type eq 'phot') then P10_button_sensitize, Event, 6, 1
	if(group_type eq 'pol') then P10_button_sensitize, Event, 15, 1
	;
end
;
;--------------------------------------------------------------------------------------
pro P10_Determine_Normalizations, Event, group_type
	;
	;	find proper normalization for each flat image
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	a_ptr = PTR_struct.PTR_Ave_RAW1
	ff_ptr = PTR_struct.PTR_Flat
	;
	MSP_BDP_print_text, Event, "Determine Normalizations requested"
	;
	deleted_data_value = -1.0d6
	;
	;	look in proper directory for differenced images
	;
	if(group_type eq 'phot') then begin
		dir = (*s_ptr).root_reduced + '10_Flat_Field_Images' + PATH_SEP() + 'difference_images' + PATH_SEP()
	endif else begin
		dir = (*s_ptr).root_reduced + '10_Flat_Field_Images' + PATH_SEP() + 'polar_difference_images' + PATH_SEP()
	endelse
	;
	test = FILE_INFO(dir)
	if(NOT(test.exists) or NOT(test.directory)) then begin
		MSP_BDP_print_text, Event, "Error - could not find difference image directory. Skipping"
		goto, P10DN_end
	endif
	;
	;	how many files are in the indicated directory?
	;
	result = FILE_SEARCH(dir+'*.fits',  /FULLY_QUALIFY_PATH, count=count)
	;
	if(count ne 0) then begin
		n_images = count
		(*ff_ptr).n_match = count
		;
		(*ff_ptr).difference_pathname[0:n_images-1] = result[*]
		(*ff_ptr).match[1,0:n_images-1] = 0
	endif else begin
		MSP_BDP_print_text, Event, 'Error - difference image directory is empty. Skipping'
		goto, P10DN_end
	endelse
	;
	;print, "number of matches = ",n_images
	;
	;	for each image
	;
	for i = 0, n_images-1 do begin
		;
		pathname = (*ff_ptr).difference_pathname[i]
		filename = FILE_BASENAME(pathname)
		;print, "reading image ", FILE_BASENAME(pathname)
		image = readfits(pathname, header, /silent)
		;
		;	parse the FITS header
		;
		n_header_lines = N_ELEMENTS(header)
		(*s_ptr).raw_header[0:n_header_lines-1] = header
		;
		MSP_BDP_Parse_FITS_Header, Event
		;
		;	classify filters, cameras
		;
		MSP_BDP_Camera_Classify, Event, this_camera
		MSP_BDP_Filter_Classify, Event, this_camera, this_filter
		;MSP_BDP_Camera_Classify, Event, this_camera
		MSP_BDP_Observation_Classify, Event, this_filter, this_observation
		;
		;	and HWP angle
		;
		MSP_BDP_Parse_HWP_Angle, Event, HWP_angle
		(*ff_ptr).HWP_angle[i] = HWP_angle
		;
;		print, "For image ",i,' filename = ',filename
;		print, '  Filter = ',this_filter,', Camera = ',this_camera,', Obs Type = ',this_observation,$
;		       ', HWP = ',HWP_angle
		;
		;	test for some key words in the filter and camera buffers
		;
		test_img = STRPOS(STRUPCASE(this_filter),'IMG') ne -1
		test_pol = STRPOS(STRUPCASE(this_filter),'POL') ne -1
		test_spec = STRPOS(STRUPCASE(this_filter),'SPEC') ne -1
		test_F5 = STRPOS(STRUPCASE(this_camera),'F5') ne -1
		test_F17 = STRPOS(STRUPCASE(this_camera),'F17') ne -1
		;
;		print, '   tests: img=',test_img,', pol=',test_pol,', spec=',test_spec,', F5=',test_F5,', F17=',test_F17
		;
		;pathname = (*ff_ptr).difference_pathname[i]
		;
		xx = [0,0]
		;
		if(test_img or test_pol and NOT(test_spec)) then begin
			;
			;	set image limits for histo calc
			;
;			xx = [100, 900]
;			yy = [100, 900]
;			dx = 10
;			dy = 10
			;
			;	reset to center zone - DPC - 20091217
			;
			xx = [362, 658]
			yy = [362, 658]
			dx = 3				; to equally sample rows, columns in readouts
			dy = 3
			;
		endif else begin
			if(test_spec) then begin
				;
				;	what type?
				;
				test_JHK = STRPOS(STRUPCASE(this_filter),'JHK-SPEC') ne -1
				test_K = (STRPOS(STRUPCASE(this_filter),'K-SPEC') ne -1) and (STRPOS(STRUPCASE(this_filter),'JH') eq -1)
				test_HK = (STRPOS(STRUPCASE(this_filter),'HK-SPEC') ne -1) and (STRPOS(STRUPCASE(this_filter),'J') eq -1)
				test_J = STRPOS(STRUPCASE(this_filter),'J-SPEC') ne -1
				test_K17 = STRPOS(STRUPCASE(this_filter),'K17-SPEC') ne -1
				test_H17 = STRPOS(STRUPCASE(this_filter),'H17-SPEC') ne -1
				test_IZYJ = STRPOS(STRUPCASE(this_filter),'IZYJ-SPEC') ne -1
				test_IZYJ_X = STRPOS(STRUPCASE(this_filter),'IZYJ-X-SPEC') ne -1
				;
;				print, '  spec-tests: JHK=',test_JHK,', K=',test_K,', J=',test_J
				;
				;	this is spectroscopy - if a known type, load known histo limits
				;
				if(test_F5 and NOT(test_F17)) then begin
					if(test_JHK) then begin
						xx = [300, 830]
						yy = [270, 740]
						dx = 3
						dy = 3
					endif
					if(test_K) then begin
						xx = [640,1010]
						yy = [250, 740]
						dx = 3
						dy = 3
					endif
					if(test_J) then begin
						xx = [640, 820]
						yy = [260, 760]
						dx = 3
						dy = 3
					endif
					if(test_HK) then begin
						xx = [280, 1010]
						yy = [280, 740]
						dx = 3
						dy = 3
					endif
					if(test_IZYJ_X) then begin
						xx = [1, 1022]
						yy = [210, 610]
						dx = 3
						dy = 3
					endif
					if(test_IZYJ) then begin
						xx = [770, 992]
						yy = [270, 740]
						dx = 3
						dy = 3
					endif
				endif else begin
					if(test_F17 and NOT(test_F5)) then begin
						if(test_K17) then begin
							xx = [740, 900]
							yy = [170, 400]
							dx = 3
							dy = 3
						endif
						if(test_H17) then begin
							xx = [1, 1022]
							yy = [85, 790]
							dx = 3
							dy = 3
						endif
					endif
				endelse
			endif
		endelse
		;
		;	if xx not set, then need to get some help from user
		;
		if(xx[0] eq 0) then begin
			MSP_BDP_print_text, Event, "Need to get user to specify region for histogram. skipping for now..."
			(*ff_ptr).normal[i] = 1.0d0  ; for now
			;
;			print, ' dropped through all xx, yy settings'
			;
			goto, loop_end
		endif
		;
		;	read image and use imhist to analyze
		;
		;print, "xx, yy, dx, dy = ",xx, yy, dx, dy
		;
		sub_image = image[xx[0]:xx[1]:dx,yy[0]:yy[1]:dy]
		;
		;	and reform into a vector
		;
		size_sub = SIZE(sub_image)
		ncol = size_sub[1]
		nrow = size_sub[2]
		npix = long(ncol) * long(nrow)
		vec = REFORM(sub_image, npix)
		;
		ind = WHERE(vec ne deleted_data_value and FINITE(vec), count)
		vec = vec[ind]
		;
		zz = median_filtered_mean(vec)
		;
;		ymax = MAX(vec)
;		ymin = MIN(vec)
;		nb = 256
		;
		;	plot +/- 3 sigma
		;
		ymax = (zz[0] + 10.0*zz[1]) > 0
		ymin = (zz[0] - 10.0*zz[1]) > 0
		nb = 101								; 0.2 rms each
		;
		h = dpc_histogram(vec, MAX=ymax, MIN=ymin, NBINS=nb)
		x = ymin + (ymax - ymin) / (nb -1.) * FIndGen(nb)
		;
		;	plot
		;
		device, decomposed=0
		color_red = REFORM((*r_ptr).color_rgb[0,*])
		color_green = REFORM((*r_ptr).color_rgb[1,*])
		color_blue = REFORM((*r_ptr).color_rgb[2,*])
		;
		tvlct, color_red, color_green, color_blue, 0
		;
		if(group_type eq 'phot') then ww = 0 else ww = 1
		window_id = (*s_ptr).P10_draw_window_id[ww]
		;
		WSET, window_id
		;
		xmin = (zz[0]-2.5*zz[1]) > ymin
		xmax = (zz[0]+2.5*zz[1]) < ymax
		;
		y_ind = WHERE(x ge xmin and x le xmax, n_yind)
		if(n_yind gt 0) then begin
			yrmin = min(h[y_ind])
			yrmax = max(h[y_ind]) * 1.1
		endif else begin
			yrmin = 1 > min(h)
			yrmax = max(h)*1.1
		endelse
		;
		xti_nominal = 200.0
		;
		xt_loop:
		;
		nxticks = round((xmax - xmin) / xti_nominal)
		if(nxticks lt 3) then begin
			xti_nominal /= 2.0
			goto, xt_loop
		endif
		if(nxticks gt 6) then begin
			xti_nominal *= 2.0
			goto, xt_loop
		endif
		;
		plot, x, h, /YLOG,yrange=[yrmin, yrmax],xrange=[xmin,xmax],psym=10,$	; xstyle=1,
				color=1, background=0, thick=1, LINESTYLE=0, $
				Xtitle = "Pixel Value", Ytitle="No. Pixels", TITLE=filename, $
				;POSITION=(*b_ptr).position,/DEVICE,
				xtickinterval=xti_nominal, $
				xminor=5, ycharsize=1.25
		;
		;	show mean value
		;
		xd = [zz[0],zz[0]]
	    yd = [10^(FLOOR(ALOG10(yrmin))),max(h)]
	    ;
	    PlotS, xd, yd, linestyle=2, thick=3, color=2
	    ;
	    ;	and +/-1 sigma width
	    ;
	    xs = [zz[0]-zz[1],zz[0]+zz[1]]
	    ys = [0.5*max(h),0.5*max(h)]
	    PlotS, xs, ys, linestyle=2, thick=3, color=6

		WAIT, 3
		;
		;	save normalization factor
		;
		(*ff_ptr).normal[i] = zz[0]
		;
		;	if the normalization is a NaN, stop
		;
		if(~FINITE(zz[0])) then begin
			MSP_BDP_print_text, Event, "ERROR - normalization factor is NaN. Stopping"
			stop
		endif
		;
		loop_end:
	endfor
	;
	if(group_type eq 'phot') then begin
		P10_button_sensitize, Event, 7, 1
		P10_Draw_Done, Event, 0, TEXT='Normalizations Found'
	endif
	if(group_type eq 'pol') then begin
		P10_button_sensitize, Event, 16, 1
		P10_Draw_Done, Event, 1, TEXT='Normalizations Found'
	endif
	;
	P10DN_end:
end
;
;---------------------------------------------------------------------------
pro P10_View_Normalizations, Event, group_type
	;
	;	list normalizations in table
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	a_ptr = PTR_struct.PTR_Ave_RAW1
	ff_ptr = PTR_struct.PTR_Flat
	;
	MSP_BDP_print_text, Event, "View Normalizations requested"
	;
	n_images = (*ff_ptr).n_match
	filenames = FILE_BASENAME((*ff_ptr).difference_pathname[0:n_images-1])
	normals = (*ff_ptr).normal[0:n_images-1]
	;
	col_headers = ['_______Difference Filename_______', 'Normalization Factor']
	ncols = 2
	;
	n_lines = n_images
	format = strarr(ncols,n_lines)
	format[0:ncols-1,*] = '(a)'
	;
	color_array = strarr(ncols,n_lines)
	;
	row_headers = strarr(n_lines)
	data_line = {filename: '', normalization: ''}
	;
	data_array = REPLICATE(data_line, n_lines)			; array of structures
	;
	for i = 0, n_lines-1 do begin
			;
			row_headers[i] = 'Flt_'+STRTRIM(STRING(format='(i4.1)',i),2)
			;
			data_array[i].filename = filenames[i]
			;
			if( (*ff_ptr).match[1,i] eq -1) then begin
				data_array[i].normalization = -1.0
			endif else begin
				data_array[i].normalization = STRTRIM(STRING(format='(F12.4)',normals[i]),2)
			endelse
			;
			color_array[*,i] = "green"
			if(normals[i] lt 0.0) then color_array[*,i] = 'red' else begin
				if(normals[i] lt 1000.0) then color_array[*,i] = 'orange' else begin
					if(normals[i] lt 2000.0) then color_array[*,i] = 'yellow'
				endelse
			endelse
			;
	endfor
	;
	r_string = "Flat-Field Normalizations"
    new_struct = Dialog_Big_Table(Title="Flat-Field Normalization Table",Label=r_string, $
					Group_Leader=Event.top, Col_Headers = col_headers, $
					Format=format, Row_headers = row_headers,$
					Data_array = data_array, Font='Helvetica*16', $
					color_array=color_array)
	;
	;	load new table data back into structure
	;
	nc = N_TAGS(new_struct)
	nr = N_ELEMENTS(new_struct)
	;
	;	note - permit change of group_type, only
	;
	for ir = 0, nr-1 do begin
		d_normal = 0.0d0
		reads, new_struct[ir].normalization, d_normal
		(*ff_ptr).normal[ir] = 	d_normal
	endfor
	;
	if(group_type eq 'phot') then begin
		P10_button_sensitize, Event, 8, 1
		WSET, (*s_ptr).P10_draw_window_id[0]
		ERASE
	endif
	if(group_type eq 'pol') then begin
		P10_button_sensitize, Event, 17, 1
		WSET, (*s_ptr).P10_draw_window_id[1]
		ERASE
	endif
	;
end
;
;---------------------------------------------------------------------------
pro P10_Normalize, Event, group_type
	;
	;	perform normalization, update header, and write flat field image
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	a_ptr = PTR_struct.PTR_Ave_RAW1
	ff_ptr = PTR_struct.PTR_Flat
	;
	MSP_BDP_print_text, Event, "Perform Normalizations requested"
	;
	deleted_data_value = -1.0e6
	;
	n_images = (*ff_ptr).n_match
	pathnames = (*ff_ptr).difference_pathname[0:n_images-1]
	filenames = FILE_BASENAME(pathnames)
	normals = (*ff_ptr).normal[0:n_images-1]
	;
	for i = 0, n_images-1 do begin
		;
		if( (*ff_ptr).match[1,i] eq -1) then goto, loop_end
		;
		;	update filename
		;
		dest = filenames[i]
		STRPUT, dest, 'Flt_', 0
		filenames[i] = dest
		;
		;	fetch difference image
		;
		image = readfits(pathnames[i],header,/silent)
		;
		;	normalize, but pay attention to bad and NaN pixels
		;
		ind = WHERE(image eq deleted_data_value or ~FINITE(image),count)
		image = image / normals[i]
		;
		if(count ne 0) then image[ind] = deleted_data_value
		;
		;	create output pathname
		;
		this_pathname = (*s_ptr).root_reduced + '10_Flat_Field_Images' + PATH_SEP() + filenames[i]
		;
		;	update header variables
		;
		FXADDPAR, header, "HISTORY", "Normalized by factor = "+STRING(format='(F12.4)',normals[i])+' on '+SYSTIME()
		FXADDPAR, header, "HISTORY","   reduced by "+(*s_ptr).who
		FXADDPAR, header, "FILENAME", filenames[i], "Created by MSP-BDP Part 10 (FF)"
		FXADDPAR, header, "DIRNAME", FILE_DIRNAME(this_pathname),"Created by MSP-BDP Part 10 (FF)"
		if(group_type eq 'pol') then begin
			FXADDPAR, header, "HWPANGLE", normal_angle((*ff_ptr).HWP_angle[i]), "Raw HWP Angle"
		endif
		;
		writefits, this_pathname, image, header
		;
		MSP_BDP_print_text, Event, "Wrote file "+filenames[i]+' to disk'
		;
		loop_end:
	endfor
	;
	if(group_type eq 'phot') then begin
		P10_Draw_Done, Event, 0, TEXT='Normalizations Applied'
	endif
	if(group_type eq 'pol') then begin
		P10_Draw_Done, Event, 1, TEXT='Normalizations Applied'
		;
		;	turn on zero hop test button
		;
		P10_Button_Sensitize, Event, 18, 1
	endif
	;
	P10_end, Event
	;
end
;
;---------------------------------------------------------------------------
pro P10_end, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	a_ptr = PTR_struct.PTR_Ave_RAW1
	ff_ptr = PTR_struct.PTR_Flat
	;
	;	free the difference pointer
	;
	PTR_FREE, (*ff_ptr).fm_ptr
	;
	;
	MSP_BDP_print_text, Event, "Flat Field Creation Completed"
	;
end
;
;--------------------------------------
pro P10_Do_Zero_Hop_Test, Event, group_type
	;
	;	parses polar flats into date groups, then tests each group for the presence
	;	of the zero phase hwp hop problem and reports findings. If any found with
	;	the hop problem, turns on button to allow fixing of the problem
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	a_ptr = PTR_struct.PTR_Ave_RAW1
	ff_ptr = PTR_struct.PTR_Flat
	;
	MSP_BDP_print_text, Event, "Test Polar Flats for Zero Hop problem requested"
	;
	P10_Draw_Done, Event, 1, TEXT='Zero Hop Testing - see below', /NO_DONE
	;
	;	clear textbox
	;
	WIDGET_CONTROL, (*s_ptr).P10_text_wid, SET_VALUE='  '
	;
	deleted_data_value = -1.0e6
	;
	;	1. how many Polar flat groups are present?
	;
	dir_test = (*s_ptr).root_reduced + '10_Flat_Field_Images' + PATH_SEP() + 'Flt_*_HWP_*'
	pol_list = FILE_SEARCH(dir_test)
	;
	;	if none, will get back null scalar string
	;
	n_files = N_ELEMENTS(pol_list)
	if(n_files eq 1) then begin
		len = STRLEN(pol_list)
		;
		;	if len zero, then no files, jump out
		MSP_BDP_print_text, Event, "ERROR - found no polar flats in flat field directory"
		MSP_BDP_print_text, Event, "processing of pol flats ceases... :(  "
		goto, no_files
	endif
	;
	MSP_BDP_print_text, Event, "Found a total of "+STRING(n_files)+" polar flats in directory"
	;
	;
	;	should parse nicely into groups of 16
	;
	n_pol_flat_groups = ROUND(n_files / 16)
	test_n = 16 * n_pol_flat_groups
	if(n_files ne test_n) then begin
		MSP_BDP_print_text, Event, "ERROR - polar flats do not parse nicely into groups of 16"
		MSP_BDP_print_text, Event, "        found number of pol-flat groups = "+STRING(n_pol_flat_groups)
		MSP_BDP_print_text, Event, "processing of pol flats ceases... :(  "
		goto, no_files
	endif
	;
	MSP_BDP_print_text, Event, 'Found a total of '+STRING(n_pol_flat_groups,format='(I)')+' distinct pol flat groups'
	;
	;	update variables in ff structure
	;
	(*ff_ptr).n_pol_groups = n_pol_flat_groups
	(*ff_ptr).zero_hop_fix_status[0:n_pol_flat_groups-1] = -1		; meaning awaiting testing
	(*ff_ptr).pol_group_ymd[0:n_pol_flat_groups-1] = 'no group here...'
	;
	;	collect files names for each group from pol_list
	;
	group_list = strarr(n_pol_flat_groups)
	;
	;	parse by filter
	;
	j_pol_list = FILE_SEARCH((*s_ptr).root_reduced + '10_Flat_Field_Images' + PATH_SEP() + 'Flt_*_J-Pol_*_HWP_*',$
					COUNT=n_j_pol_list)
	h_pol_list = FILE_SEARCH((*s_ptr).root_reduced + '10_Flat_Field_Images' + PATH_SEP() + 'Flt_*_H-Pol_*_HWP_*',$
					COUNT=n_h_pol_list)
	k_pol_list = FILE_SEARCH((*s_ptr).root_reduced + '10_Flat_Field_Images' + PATH_SEP() + 'Flt_*_K-Pol_*_HWP_*',$
					COUNT=n_k_pol_list)
	l_pol_list = FILE_SEARCH((*s_ptr).root_reduced + '10_Flat_Field_Images' + PATH_SEP() + 'Flt_*_L-Pol_*_HWP_*',$
					COUNT=n_l_pol_list)
	m_pol_list = FILE_SEARCH((*s_ptr).root_reduced + '10_Flat_Field_Images' + PATH_SEP() + 'Flt_*_M-Pol_*_HWP_*',$
					COUNT=n_m_pol_list)
	;
	;	for each potential polar flag filter group,
	;
	ipfgroup = 0
	;
	for ifilt = 0, 4 do begin
		case ifilt of
			0:	begin
					n_list = n_j_pol_list
					if(n_list gt 0) then pol_list = j_pol_list
					filt_string = 'J'
					zht_0 = 65.0			; guess
					zht_1 = 15.0			; guess
				end
			1:	begin
					n_list = n_h_pol_list
					if(n_list gt 0) then pol_list = h_pol_list
					filt_string = 'H'
					zht_0 = 65.0			; measured
					zht_1 = 15.0			; measured
				end
			2:	begin
					n_list = n_k_pol_list
					if(n_list gt 0) then pol_list = k_pol_list
					filt_string = 'K'
					zht_0 = 135.0			; first cut 20111030
					zht_1 = 85.0			; "
				end
			3:	begin
					n_list = n_l_pol_list
					if(n_list gt 0) then pol_list = l_pol_list
					filt_string = 'L'
					zht_0 = 65.0			; guess
					zht_1 = 15.0			; guess
				end
			4:	begin
					n_list = n_m_pol_list
					if(n_list gt 0) then pol_list = m_pol_list
					filt_string = 'M'
					zht_0 = 65.0			; guess
					zht_1 = 15.0			; guess
				end
			else: begin
					MSP_BDP_print_text, Event, 'ERROR - did not recognize flat groups'
					MSP_BDP_print_text, Event, "processing of pol flats ceases... :(  "
					goto, no_files
				  end
		endcase
		;
		;	if none, dump out
		;
		if(n_list eq 0) then goto, filt_loop_end
		;
		;	also, if n_list > 16 issue error
		;
		if(n_list gt 16) then begin
			MSP_BDP_print_text, Event, 'ERROR - too many pol flats with same filter '+filt_string+' this night'
			MSP_BDP_print_text, Event, "processing of this filter pol flat ceases... :(  "
			goto, filt_loop_end
		endif
		;
		;for ipfgroup = 0, n_pol_flat_groups-1 do begin
		;
		;	pull first file off the list and use its embedded date name to limit search
		;
		first = STRPOS(pol_list[0],'Flt_')
		;
		date_str = STRTRIM(STRMID(pol_list[0],first+4,8),2)
		group_list[ipfgroup] = date_str
		;
		MSP_BDP_print_text, Event, "Collecting "+filt_string+"-pol flats for group "+date_str
		;
		new_test = (*s_ptr).root_reduced + '10_Flat_Field_Images' + PATH_SEP() + 'Flt_'+date_str+'_'+$
										filt_string+ '-Pol_*_HWP_*'
		this_pol_list = FILE_SEARCH(new_test)
		;
		;	did we get 16?
		;
		this_test = N_ELEMENTS(this_pol_list)
		if(this_test ne 16) then begin
			MSP_BDP_print_text, Event, "ERROR - got "+STRING(this_test)+" pol flats, where 16 were expected"
			MSP_BDP_print_text, Event, "processing of this filter pol flats ceases... :(  "
			goto, filt_loop_end
		endif
		;
		;	yes - remove them from the old list - which automatically pushes the old list down so it
		;			is ready for the next loop on the groups
		;
;		for i = 0, 15 do begin
;			ind = WHERE(pol_list ne this_pol_list[i],no_match)
;			if(no_match gt 0) then begin
;				pol_list = pol_list[ind]
;			endif
;		endfor
		;
		;	test this group of pol flats for the Zero Hop problem
		;
		;	first, sort them
		;
		file_list = FILE_BASENAME(STRUPCASE(this_pol_list),'.FITS')
		;
		n_files = N_ELEMENTS(file_list)
		scan_numbers = lonarr(n_files)
		;
		flat_flat = 0
		for i = 0, n_files-1 do begin
			;
			;	parse name to get sequence number
			;
			;		first, flat or CDS?
			;
			this_type = STRPOS(file_list[i],'FLT')
			if(this_type ne -1) then begin
				;
				;	was flat, create artificial scan numbers
				;
				last = STRLEN(file_list[i])
				first = last - 4
				num_str = STRMID(file_list[i],first,(last-first))
				reads, num_str, format='(I)',scan
				scan_numbers[i] = ROUND(float(scan) / 75.0)
				;print, "Flat - i, num_str, scan_number = ",i, num_str, scan_numbers[i]
				;
				flat_flag = 1
			endif else begin
				first = STRPOS(file_list[i],'.') + 1
				last = STRLEN(file_list[i])
				num_str = STRMID(file_list[i],first,(last-first))
				reads, num_str, format='(I)',scan
				scan_numbers[i] = scan
			endelse
		endfor
		;
		ind_scan = SORT(scan_numbers)
		files_in = this_pol_list[ind_scan]
		;
		;print, "sorted scan numbers = ",scan_numbers[ind_scan]
		;
		;	ok, sorted, now test
		;
		this_debug = 0
		;
		check_dither = 1 - flat_flag
		;
		;	turn on hourglass
		;
		WIDGET_CONTROL, /HOURGLASS
		;
		hwp_fidelity_test, Event, files_in, check_dither, fidelity_out, fold_angle, $
						mean_PA, pa_dith, s_pa_dith, dither_vector, this_debug, DITHER=my_dither
		;
		;	test for whether a fix is needed
		;
		;	NB: the 65 and 15 deg values pertain to the H-HWP - if other HWPs get implemented,
		;			a look-up table will be needed  (DPC 20091223)
		;
		if(abs(abs(mean_PA)-zht_0) lt abs(abs(mean_PA)-zht_1)) then begin
			;
			;	this pol group is fine and needs no fix
			;
			MSP_BDP_print_text, Event, filt_string+"-Pol Group "+group_list[ipfgroup]+" has no Zero Hop problem - fix not needed"
			MSP_BDP_print_text, Event, "  its nominal PA was "+STRING(mean_PA)+" degrees"
			;
			;	write message to textbox for user to see
			;
			WIDGET_CONTROL, (*s_ptr).P10_text_wid, /APPEND, SET_VALUE='Grp '+filt_string+group_list[ipfgroup]
			WIDGET_CONTROL, (*s_ptr).P10_text_wid, /APPEND, SET_VALUE='   - no Zero Hop'
			;
			;	save "no fix" flag in ff structure
			;
			(*ff_ptr).zero_hop_fix_status[ipfgroup] = 1			; means no fix needed
			(*ff_ptr).pol_group_ymd[ipfgroup] = date_str		; save group name (YYYYMMDD)
			(*ff_ptr).pol_group_filter[ipfgroup] = filt_string
			;
		endif else begin
			;
			;	this pol group needs to be fixed
			;
			MSP_BDP_print_text, Event, filt_string+"-Pol Group "+group_list[ipfgroup]+" exhibits Zero Hop problem - fix needed"
			MSP_BDP_print_text, Event, "  its nominal PA was "+STRING(mean_PA)+" degrees"
			;
			;	write message to textbox for user to see
			;
			WIDGET_CONTROL, (*s_ptr).P10_text_wid, /APPEND, SET_VALUE='Grp '+filt_string+group_list[ipfgroup]
			WIDGET_CONTROL, (*s_ptr).P10_text_wid, /APPEND, SET_VALUE='   - Zero Hop !!!'
			;
			;	save "to be fixed" flag in ff structure
			;
			(*ff_ptr).zero_hop_fix_status[ipfgroup] = 2			; means fix needed
			(*ff_ptr).pol_group_ymd[ipfgroup] = date_str
			(*ff_ptr).pol_group_filter[ipfgroup] = filt_string
		endelse
		;
		;	save the mean PA for the group in the FF directory, tagged by filter
		;
		ff_dir = FILE_DIRNAME(files_in[0])
		;
		header_flat = headfits(files_in[0], /silent)
		n_header_lines = N_ELEMENTS(header_flat)
		(*s_ptr).raw_header[0:n_header_lines-1] = header_flat
		(*s_ptr).raw_header[n_header_lines:N_ELEMENTS((*s_ptr).raw_header)-1] = ''
		MSP_BDP_Parse_FITS_Header, Event
		MSP_BDP_Camera_Classify, Event, camera_summary_name
		MSP_BDP_Filter_Classify, Event, camera_summary_name, filter_summary_name
		;
		ff_path = ff_dir + PATH_SEP() + filter_summary_name + '_FF_'+date_str+'_mean_PA.dat'
		;
		openw, lu_pa, ff_path, /GET_LUN
		printf, lu_pa, '; mean PA for '+filt_string+'-Pol FF group ',date_str,' computed ',SYSTIME()
		if((*ff_ptr).zero_hop_fix_status[ipfgroup] eq 1) then begin
			printf, lu_pa, '; this group does not need to be fixed via rotation'
		endif else begin
			printf, lu_pa, '; this group NEEDS to be fixed via rotation'
		endelse
		printf, lu_pa, format='("MEAN_PA =                ",F9.3," / Degrees")',mean_PA
		FREE_LUN, lu_pa
		;
		;	increment the group counter and test next group
		;
		ipfgroup++
		;
		;	come here if none processed with indicated filter
		;
		filt_loop_end:
	endfor
	;
	;	if any were found to need fixing, turn on fix action button
	;
	ind = WHERE( (*ff_ptr).zero_hop_fix_status[0:n_pol_flat_groups-1] eq 2, nfix)
	if(nfix gt 0) then begin
		;
		MSP_BDP_print_text, Event, "Found "+STRING(nfix)+" polar groups need Zero Hop fix applied"
		;
		P10_button_sensitize, Event, 19, 1
	endif
	;
	no_files:
	;
	MSP_BDP_print_text, Event, "Testing of Polar Flats for Zero Hop Problem finished"
end
;
;-------------------------------------------
pro P10_Fix_Zero_Hops, Event, group_type
	;
	;	for each pol group needing zero hop fixed, saves old files
	;	in a new subdirectory, then uses those files to create new
	;	phase delayed files
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	a_ptr = PTR_struct.PTR_Ave_RAW1
	ff_ptr = PTR_struct.PTR_Flat
	;
	MSP_BDP_print_text, Event, "Fix of Polar Flats for Zero Hop problem requested"
	;
	P10_Draw_Done, Event, 1, TEXT='Fixing Zero Hops', /NO_DONE
	;
	deleted_data_value = -1.0e6
	;
	;	how many polar groups exist?
	;
	n_pol_groups = (*ff_ptr).n_pol_groups
	;
	;	how many need fixing?
	;
	ind_fix = WHERE( (*ff_ptr).zero_hop_fix_status[0:n_pol_groups-1] eq 2, nfix)
	MSP_BDP_print_text, Event, "found "+STRING(nfix)+" groups on list needing fix"
	;
	for igrp = 0, nfix-1 do begin
		;
		this_group = ind_fix[igrp]
		pol_group_YMD = (*ff_ptr).pol_group_ymd[this_group]
		this_filter = (*ff_ptr).pol_group_filter[this_group]
		;
		;	build list of the existing images
		;
		MSP_BDP_print_text, Event, "Collecting "+this_filter+"-Pol flats for group "+pol_group_YMD
		;
		new_test = (*s_ptr).root_reduced + '10_Flat_Field_Images' + PATH_SEP() + 'Flt_'+pol_group_YMD+$
									'_' + this_filter + '-Pol_*_HWP_*'
		this_pol_list = FILE_SEARCH(new_test)
		;
		;	did we get 16?
		;
		this_test = N_ELEMENTS(this_pol_list)
		if(this_test ne 16) then begin
			MSP_BDP_print_text, Event, "ERROR - got "+STRING(this_test)+" pol flats, where 16 were expected"
			MSP_BDP_print_text, Event, "processing of pol flats ceases... :(  "
			goto, fix_no_files
		endif
		;
		;	create a new subdirectory
		;
		this_date = SYSTIME()
		new_date = bdp_replace_underscores(this_date)
		sub_dir = FILE_DIRNAME(this_pol_list[0]) + PATH_SEP() + 'Polar_Originals_' + pol_group_YMD + '_on_' + new_date
		;
		FILE_MKDIR, sub_dir
		;
		;	and copy the files
		;
		out_paths = fix_path( sub_dir + PATH_SEP() + FILE_BASENAME(this_pol_list) )
		;
		FILE_COPY, this_pol_list, sub_dir, /REQUIRE_DIRECTORY
		;
		MSP_BDP_print_text, Event, "Original Pol Flat files successfully copied to directory:"
		MSP_BDP_print_text, Event, "  "+sub_dir
		;
		;	delete files from current directory
		;
		FILE_DELETE, this_pol_list
		;
		;	next - do phase shift and write to current directory
		;
		;	fetch the current mean_PA off the disk file, if present - if not present, recompute it
		;
		ff_dir = FILE_DIRNAME(this_pol_list[0])
		;
		ff_path = ff_dir + PATH_SEP() + this_filter+ '-Pol_FF_'+pol_group_YMD+'_mean_PA.dat'
		;
		f_test = FILE_INFO(ff_path)
		if(f_test.exists) then begin
			openr, lu_pa, ff_path, /GET_LUN
			line = ''
			WHILE ~EOF(lu_pa) do begin
				readf, lu_pa, line
				indx = STRPOS(line, ';')
				;
				;	skip lines beginning with semicolons
				;
				if(indx eq -1) then begin
					last = STRPOS(line,'/')
					short_line = STRMID(line, 9, (last-9))
					reads, short_line, mean_PA
				endif
			ENDWHILE
			FREE_LUN, lu_pa
		endif else begin
			;
			;	no file, write problem to screen and kick out
			;
			MSP_BDP_print_text, Event, "ERROR - must test group "+pol_group_YMD+" for Zero Hop before correction"
			goto, fix_no_files
		endelse
		;
		;	note that the offset phase angle of 25.0 deg is for the H-HWP, only.
		;	when/if more HWPs get installed, will need to use a look-up table to
		;	get correct offset
		;
		offset_angle = 25.0		; this is based on zero hopping, which shouldn't affect K-band, so leave in place 20111030 dpc
		;
		;	turn on hourglass
		;
		WIDGET_CONTROL, /HOURGLASS
		;
		retard_flat_phase, out_paths, this_pol_list, offset_angle, ier
		;
		if(ier ne 0) then begin
			MSP_BDP_print_text, Event, "ERROR - Pol Zero Hop Fix failed with IER = "+STRING(ier)+" for group "+pol_group_YMD
		endif else begin
			MSP_BDP_print_text, Event, "Pol Zero Hop Fixed for group "+pol_group_YMD
			;
			;	change its status on the ff structure list to "fixed"
			;
			(*ff_ptr).zero_hop_fix_status[this_group] = 1
			;
			;	delete, the rewrite the summary file
			;
			FILE_DELETE, ff_path
			;
			openw, lu_pa, ff_path, /GET_LUN
			printf, lu_pa, '; mean PA for '+this_filter+'-Pol FF group ',pol_group_YMD,' computed ',SYSTIME()
			printf, lu_pa, '; this group was fixed via rotation'
			printf, lu_pa, format='("MEAN_PA =                ",F9.3," / Degrees")',mean_PA+(2.0*offset_angle)
			FREE_LUN, lu_pa
			;
		endelse
		;
		;	next group
		;
	endfor
	;
	P10_Draw_Done, Event, 1, TEXT='Hops all fixed!'
	;
	fix_no_files:
end
;
;-------------------------------------------------------------------------
pro P10_Draw_Done, Event, win_index, TEXT=text, NO_DONE=no_done
	;
	;	clears draw space and writes "Done !" in center of draw window win_index
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	b_ptr = PTR_struct.PTR_BPM
	;
	WSET, (*s_ptr).P10_draw_window_id[win_index]
	ERASE
	;
	if(N_ELEMENTS(no_done) eq 0) then no_done = 0 else no_done = 1
	;
	device, decomposed=0
	color_red = REFORM((*r_ptr).color_rgb[0,*])
	color_green = REFORM((*r_ptr).color_rgb[1,*])
	color_blue = REFORM((*r_ptr).color_rgb[2,*])
	;
	tvlct, color_red, color_green, color_blue, 0
	;
	window_id = (*s_ptr).P10_draw_window_id[win_index]
	;
	WSET, window_id
	;
	x = [0., 1.]
	y = [0., 1.]
	;
	plot, x, y, color=0, xrange=[0.,1.], yrange=[0.,1.], xstyle=1, ystyle=1, $
				background=0, thick=1, LINESTYLE=0, $
				/DEVICE,/NODATA
	;
	if(no_done eq 0) then XYoutS, 0.5, 0.5, "Done !", ALIGNMENT=0.5, color=2, charthick=2, charsize=2.0
	;
	nt = N_ELEMENTS(TEXT)
	if(nt eq 1) then XYoutS, 0.5, 0.25, text, ALIGNMENT=0.5, color=2, charthick=2, charsize=1.5
	;
end
;
;--------------------------------------------------------------------------------------------
pro P10_Save_Night_Summary, Event, group_type
	;
	;	saves an updated night summary
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	a_ptr = PTR_struct.PTR_Ave_RAW1
	ff_ptr = PTR_struct.PTR_Flat
	;
	;	for each night in the dark list, write the number of groups coadded
	;
	if(group_type eq 'phot') then begin
		n_groups = (*ff_ptr).n_flat_groups
		n_nights = (*ff_ptr).n_nights
		;
		n_darks = intarr(n_nights)
		;
		for ig = 0, n_groups-1 do begin
			ind = WHERE( (*ff_ptr).night_desig[ig] eq (*ff_ptr).night_list[0:n_nights-1],count)
			if(count eq 1) then n_darks[ind[0]]++
		endfor
		;
		;	for each possible night
		;
		for inight = 0, n_nights-1 do begin
			;
			;	if there were non-zero number of flat images made, write to summary file for this night
			;
			if(n_darks[inight] ne 0) then begin
				;
				this_night = (*ff_ptr).night_list[inight]
				;
				;	which one in the summary?
				;
				num_nights = (*s_ptr).num_nights
				ind = WHERE(this_night eq (*s_ptr).night_date[0:num_nights-1], count)
				if(count ne 0) then night_index = ind[0] else begin
					MSP_BDP_print_text, Event, "Error - No matching night in P10..."
					goto, P10SNS_end
				endelse
				;
				(*s_ptr).night_who[night_index] = (*s_ptr).who
				(*s_ptr).night_step[night_index] = 10
				text = STRTRIM(STRING(format='(i4.1)',n_darks[inight]),2)+" Co-Adds Made"
				if(group_type eq 'phot') then begin
					text = 'I-,S-' + text
				endif else begin
					text = 'P-' + text
				endelse
				(*s_ptr).night_comment[night_index] = text
				;
				P1_Write_Summary_File, Event
				;
			endif
		endfor
	endif else begin
		;
		;	pol case
		;
		;	for every pol flat set
		;
		n_polar_flats = (*ff_ptr).n_polar_flats
		for ipf = 0, n_polar_flats-1 do begin
			;
			;	select the night
			;
			this_night = (*ff_ptr).polar_flat[ipf].polar_group[0].night_desig
			;
			;	which one in the summary?
			;
			num_nights = (*s_ptr).num_nights
			ind = WHERE(this_night eq (*s_ptr).night_date[0:num_nights-1], count)
			if(count ne 0) then night_index = ind[0] else begin
				MSP_BDP_print_text, Event, "Error - No matching night in P10..."
				goto, P10SNS_end
			endelse
			;
			(*s_ptr).night_who[night_index] = (*s_ptr).who
			(*s_ptr).night_step[night_index] = 10
			;
			text = "Polar Co-Adds Made"
			(*s_ptr).night_comment[night_index] = text
		endfor
		;
		P1_Write_Summary_File, Event
	endelse
	;
	P10SNS_end:
end
