;
;	Part 11 Event Handler Routines - Measure Readout Time
;
;-----------------------------------------------------------------
;
pro P11_Discover_Linearity_Images, Event
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	rt_ptr = PTR_struct.PTR_RTime
	;
	;	look for coadded linearity images
	;
	MSP_BDP_print_text, Event, "Discover Linearity Images Requested"
	;
	lin_dir = (*s_ptr).root_reduced + '6_Linearity_Tables' + PATH_SEP()
	;
	;	does this directory exist? and what is in it?
	;
	test = FILE_INFO(lin_dir)
	if(NOT(test.exists) or NOT(test.directory)) then begin
		MSP_BDP_print_text, Event, "ERROR - No Linearity Images to analyze. Skipping."
		goto, P11DLI_end
	endif
	;
	;		get list of what is there
	;
	list = FILE_SEARCH(lin_dir+'*',/MARK_DIRECTORY,COUNT=n_list)
	if(n_list eq 0) then begin
		MSP_BDP_print_text, Event, "Linearity Tables directory is empty. Skipping."
		goto, P11DLI_end
	endif
	;
	;		for each subdirectory, verify that there is something there.
	;
	i_found = 0
	save_pathname_list = ''
	;
	for id = 0, n_list-1 do begin
		;
		;	look for method 2 linearity first
		;
		test_dir = list[id] + 'coadded_images_method_2' + PATH_SEP()
		test = FILE_INFO(test_dir)
		if(test.exists and test.directory) then begin
			;
			;	any files there?
			;
			files = FILE_SEARCH(test_dir, '*.fits', COUNT=f_count)
			if(f_count ne 0) then begin
				;
				;	ok there were files here. mark found and save the list
				;
				i_found=1
				save_pathname_list = files
			endif
		endif
		;
		;	if we found one, jump out -- only need one at this point
		;		(may have to change this when subframing)
		;
		if(i_found) then goto, out_of_loop
		;
		;	if not, then look for method 1 linearity second
		;
		test_dir = list[id] + 'coadded_images' + PATH_SEP()
		test = FILE_INFO(test_dir)
		if(test.exists and test.directory) then begin
			;
			;	any files there?
			;
			files = FILE_SEARCH(test_dir, '*.fits', COUNT=f_count)
			if(f_count ne 0) then begin
				;
				;	ok there were files here. mark found and save the list
				;
				i_found=1
				save_pathname_list = files
			endif
		endif
		;
		if(i_found) then goto, out_of_loop
	endfor
	if(NOT(i_found)) then begin
		MSP_BDP_print_text, Event, "No Coadded Linearity Images found. Skipping;"
		goto, P11DLI_end
	endif
	;
	out_of_loop:
	;
	;	ok, we have a list. next, remove the bias item and then keep only first 15 items on the list
	;
	ind = WHERE(STRPOS(save_pathname_list[*], 'Bias') lt 0,count)
	if(count gt 10) then count = 5
	save_pathname_list = save_pathname_list[ind[1:count]]
	;
	(*rt_ptr).n_list = count
	(*rt_ptr).path_list[0:count-1] = save_pathname_list[*]
	(*rt_ptr).status[0:count-1] = 'Include'
	;
	MSP_BDP_print_text, Event, "Put "+STRING(format='(I4.1)',count)+' coadded linearity images on list'
	;
	P11_button_sensitize, Event, 1, 1
	P11_button_sensitize, Event, 3, 1
	;
	P11DLI_end:
end
;
;------------------------------------------------------
pro P11_View_Edit_Linearity_List, Event
	;
	;	show the linearity list
	;
	MSP_BDP_print_text, Event, "Show/Edit List of Coadded Linearity Images requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	s_ptr = PTR_struct.PTR_summary
	n_ptr = PTR_struct.PTR_night
	r_ptr = PTR_struct.PTR_RAW1
	rt_ptr = PTR_struct.PTR_RTime
	;
	n_images = (*rt_ptr).n_list
	;
	;------------------------------------------------------------------------------------
	;
	;	build the table structure
	;
	col_headers = ['___Image Filename___', '__Status__']
	ncols = N_ELEMENTS(col_headers)
	;
	n_lines = n_images
	format = strarr(ncols,n_lines)
	format[0:ncols-1,*] = '(a)'
	;
	color_array = strarr(ncols,n_lines)
	;
	row_headers = strarr(n_lines)
	data_line = {filename: '', status: ''}
	;
	data_array = REPLICATE(data_line, n_lines)			; array of structures
	;
	for i = 0, n_lines-1 do begin
			;
			row_headers[i] = 'Lin_'+STRTRIM(STRING(format='(i4.1)',i),2)
			;
			data_array[i].filename = FILE_BASENAME((*rt_ptr).path_list[i])
			data_array[i].status    = (*rt_ptr).status[i]
			;
			case data_array[i].status of
				"Skip":color = 'red'
				"**BAD**":	color = "red"
				"Include": color = "green"
				"Exclude": color = "red"
					else: color = "yellow"
			endcase
			;
			color_array[*,i] = color
	endfor
	;
	r_string = "Coadded Linearity Image List" + "   or exclude by putting 'Skip' in the Status column entry"
    new_struct = Dialog_Big_Table(Title="Coadded Linearity List",Label=r_string, $
					Group_Leader=Event.top, Col_Headers = col_headers, $
					Format=format, Row_headers = row_headers,$
					Data_array = data_array, Font='Helvetica*16', $
					color_array=color_array)
	;
	;	load new table data back into structure
	;
	nc = N_TAGS(new_struct)
	nr = N_ELEMENTS(new_struct)
	;
	;	note - permit change of status only
	;
	for ir = 0, nr-1 do begin
		if( new_struct[ir].status ne 'Include') then begin
			(*rt_ptr).status[ir] = 'Skip'
		endif
	endfor
	;
	P11_ST_end:
end
;
;---------------------------------------------------
pro P11_Linearity_Correct, Event
	;
	;	Linearity corrects the coadded linearity images
	;
	MSP_BDP_print_text, Event, "Linearity Correct Image List requested
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	rt_ptr = PTR_struct.PTR_RTime
	n_ptr = PTR_struct.PTR_night
	b_ptr = PTR_struct.PTR_BPM
	a_ptr = PTR_struct.PTR_Ave_RAW1
	f_ptr = PTR_struct.PTR_FITS
	lc_ptr = PTR_struct.PTR_LC
	;
	;	first load the linearity tables existing
	;
	P8_View_Linearity_Tables, Event
	;
	;	number of images to process
	;
	n_groups = (*rt_ptr).n_list
	n_good = 0L
	for ig = 0, n_groups-1 do begin
		if( STRUPCASE( (*rt_ptr).status[ig]) ne 'SKIP') then n_good++
	endfor
	;
	;	clear LED progress bar to yellow
	;
	n_LEDs = N_ELEMENTS( (*s_ptr).P11_LED_colors )
	LED_scale = float(n_good) / float(n_LEDs)
	if(LED_scale le 1.0) then LED_scale = 1.0
	(*s_ptr).P11_LED_colors[*] = 'yellow'
	if(n_good lt n_LEDs) then (*s_ptr).P11_LED_colors[n_good:n_LEDs-1] = 'black'
	WCT_LED_stack, (*s_ptr).P11_LED_wid[0], (*s_ptr).P11_LED_colors
	;
	;	start by fetching BPM directory list
	;
	MSP_BDP_Read_BPM_Headers, Event
	;
	;	how many did we load? If none, issue message and leave
	;
	n_bpm = (*b_ptr).bpm_n_BPM
	if(n_bpm eq 0) then begin
		MSP_BDP_print_text, Event, "ERROR - No Bad Pixel Masks found - go back to Part 5"
		goto, P11SLC_outer_end
	endif
	;
	;	and the Ave RAW1 list
	;
	MSP_BDP_Read_Ave_RAW1_Headers, Event
	n_ave_raw1 = (*a_ptr).n_groups
	if(n_ave_raw1 eq 0) then begin
		MSP_BDP_print_text, Event, "ERROR - No Ave RAW1 Images found - go back to Part 7"
		goto, P11SLC_outer_end
	endif
	;
	;	load linearity table
	;
	this_lin_table_no = 1
	;
	P8_Read_Linearity_Tables, Event, this_lin_table_no
	;
	;	for every good image
	;
	i_good = 0L
	;
	for ig = 0, n_groups -1 do begin
		;
		if(STRUPCASE( (*rt_ptr).status[ig]) eq 'SKIP') then begin
			MSP_BDP_print_text, Event, 'Skipping image '+STRING(format='(I4.1)',ig)
			goto, group_loop_end
		endif
		;
		this_LED = ROUND( float(i_good) / LED_scale )
		this_LED = 0 > ((n_LEDs - 1) < this_LED)
		(*s_ptr).P11_LED_colors[this_LED] = 'orange'
		WCT_LED_stack, (*s_ptr).P11_LED_wid[0], (*s_ptr).P11_LED_colors
		;
		;	determine JD of this observation
		;
		this_path = (*rt_ptr).path_list[ig]
		;
		(*s_ptr).raw_header = HEADFITS(this_path, /silent)
		;
		MSP_BDP_Parse_FITS_Header, Event
		;
		date_obs = (*f_ptr).date_obs
		reads, date_obs, format='(i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,f5.2)',year, month, day, hour, minute, second
		;
		this_jd = JULDAY(month, day, year, hour, minute, ROUND(second))
		;
		;print, "this JD  ", this_jd
		;
		;	use obs JD to match to BPM
		;
		d_jd = abs(this_jd - (*b_ptr).bpm_JD[0:n_bpm-1])
		d_jd_min = MIN(d_jd)
		ind = WHERE(d_jd_min eq d_jd[*],count)
		if(count ne 1) then begin
			MSP_BDP_print_text, Event, "ERROR - bad BPM JD match in P11"
			goto, P11SLC_outer_end
		endif
		;
		;print, "selected BPM JD = ",(*b_ptr).bpm_JD[ind[0]]
		;
		BPM_path = (*b_ptr).bpm_image_name[ind[0]]
		;
		MSP_BDP_print_text, Event, "  BPM file selected = "+FILE_BASENAME(BPM_path)
		;
		;	same for Ave_RAW1 image
		;
		d_jd = abs(this_jd - (*a_ptr).group[0:n_ave_raw1-1].jd)
		d_jd_min = MIN(d_jd)
		ind = WHERE(d_jd_min eq d_jd[*],count)
		if(count ne 1) then begin
			MSP_BDP_print_text, Event, "ERROR - bad Ave_RAW1 JD match in P11"
			goto, P11SLC_outer_end
		endif
		;
		AR_path = (*a_ptr).group[ind[0]].image_CDS_pathname[0]
		;
		MSP_BDP_print_text, Event, "  Ave_RAW1 file selected = "+FILE_BASENAME(AR_path)
		;
		;------------------------------
		;	summarize input files
		;
		CDS_path = this_path
		exptime = ''
		reads, FILE_BASENAME(this_path), format='(4x,a8)',exptime
		RAW1_path = FILE_DIRNAME(this_path, /MARK_DIRECTORY) + 'raw1_' + exptime
		;
		;print, "CDS_path = ",CDS_path
		;print, "RAW1_path = ",RAW1_path
		;
		BPM_path = BPM_path
		AR_path = AR_path
		;
		Lin_0_path = (*l_ptr).lin_table_0_path[ this_lin_table_no -1 ]
		;
		;	output directory - test and create if necessary
		;
		out_dir = (*s_ptr).root_reduced + '11_Readout_Time' + PATH_SEP()
		test = FILE_INFO(out_dir)
		if(NOT(test.exists) or NOT(test.directory)) then FILE_MKDIR, out_dir
		;
		out_path = out_dir + 'Rdt_' + exptime + '_LC.fits'
		;
		;	linearity correction file
		;
		(*s_ptr).P11_LED_colors[this_LED] = 'blue'
		WCT_LED_stack, (*s_ptr).P11_LED_wid[0], (*s_ptr).P11_LED_colors
		;
		MSP_BDP_print_text, Event, "Starting Linearity Correction of "+FILE_BASENAME(out_path)
		;
		linearity_correct, Event, CDS_path, RAW1_path, AR_path, BPM_path, $
								 corrected_image, corrected_header
		;
		;	update header and write to disk
		;
		(*s_ptr).P11_LED_colors[this_LED] = 'violet'
		WCT_LED_stack, (*s_ptr).P11_LED_wid[0], (*s_ptr).P11_LED_colors
		;
		FXADDPAR, corrected_header, "FILENAME", FILE_BASENAME(out_path), "Linearity Corrected Image"
		FXADDPAR, corrected_header, "HISTORY","  Linearity Corrected on "+SYSTIME()
		FXADDPAR, corrected_header, "HISTORY","     reduced by "+(*s_ptr).who
		FXADDPAR, corrected_header, "HISTORY", "    using Linearity Correction images starting with:"
		FXADDPAR, corrected_header, "HISTORY", "      "+FILE_BASENAME(Lin_0_path)
		FXADDPAR, corrected_header, "HISTORY", "  and Ave RAW1 image "+FILE_BASENAME(AR_path)
		FXADDPAR, corrected_header, "HISTORY", "  and Bad Pixel Mask "+FILE_BASENAME(BPM_path)
		;
		writefits, out_path, corrected_image, corrected_header
		;
		;-----------------------------
		(*s_ptr).P11_LED_colors[this_LED] = 'green'
		WCT_LED_stack, (*s_ptr).P11_LED_wid[0], (*s_ptr).P11_LED_colors
		;
		MSP_BDP_print_text, Event,"Wrote output image "+out_path
		i_good++
		;
		group_loop_end:
	endfor
	;
	;P11_Save_Night_Summary, Event, group_type
	;
	P11_button_sensitize, Event, 3, 1
	;
	P11SLC_outer_end:
end
;
;-----------------------------------------------------------------
pro P11_Read_LC_Images, Event
	;
	MSP_BDP_print_text, Event, "Read Linearity Corrected Images requested
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	rt_ptr = PTR_struct.PTR_RTime
	n_ptr = PTR_struct.PTR_night
	;
	;	create a 3D variable to hold the images and its pointer
	;
;	rdt_dir = (*s_ptr).root_reduced + '11_Readout_Time' + PATH_SEP()
;	test = FILE_INFO(rdt_dir)
;	if(test.exists and test.directory) then begin
;		file_list = FILE_SEARCH(rdt_dir+'Rdt_*.fits',COUNT=count)
;		if(count le 0) then begin
;			MSP_BDP_print_text, Event, "ERROR - No Linearity Corrected Images in directory. Skipping."
;			goto, P11RLCI_end
;		endif
;	endif else begin
;		MSP_BDP_print_text, Event, "ERROR - Directory of Linearity Corrected Images does not exist. Skipping"
;		goto, P11RLCI_end
;	endelse
	;
	rt_count = (*rt_ptr).n_list
	;
	n_LEDs = N_ELEMENTS( (*s_ptr).P11_LED_colors )
	LED_scale = float(rt_count) / float(n_LEDs)
	if(LED_scale le 1.0) then LED_scale = 1.0
	(*s_ptr).P11_LED_colors[*] = 'yellow'
	if(rt_count lt n_LEDs) then (*s_ptr).P11_LED_colors[rt_count:n_LEDs-1] = 'black'
	WCT_LED_stack, (*s_ptr).P11_LED_wid[1], (*s_ptr).P11_LED_colors
	;
	;	list of files
	;
	file_list = (*rt_ptr).path_list[0:rt_count - 1]
	rt_status = (*rt_ptr).status[0:rt_count-1]
	;
	;	read first image to get size
	;
	(*s_ptr).P11_LED_colors[0] = 'orange'
	WCT_LED_stack, (*s_ptr).P11_LED_wid[1], (*s_ptr).P11_LED_colors
	;
	image = readfits(file_list[0],header, /silent)
	image_size = SIZE(image)
	nc = image_size[1]
	nr = image_size[2]
	;
	;	save path for later header access
	;
	(*rt_ptr).path_0 = file_list[0]
	;
	;	create 3D data array
	;
	(*rt_ptr).data_ptr = PTR_NEW(fltarr(rt_count, nc, nr))
	;
	;	and do loop of filling it
	;
	(*(*rt_ptr).data_ptr)[0,*,*] = image
	;
	;	as well as the time array
	;
	itime = 0L
	reads, FILE_BASENAME(file_list[0]), format='(4x,i8)',itime
	(*rt_ptr).exptime[0] = double(itime) / 1.0d6
	;
	(*s_ptr).P11_LED_colors[0] = 'green'
	WCT_LED_stack, (*s_ptr).P11_LED_wid[1], (*s_ptr).P11_LED_colors
	;
	for i = 1, rt_count-1 do begin
		this_LED = ROUND (float(i) / LED_scale)
		this_LED = 0 > ((n_LEDs - 1) < this_LED)
		;
		(*s_ptr).P11_LED_colors[this_LED] = 'orange'
		WCT_LED_stack, (*s_ptr).P11_LED_wid[1], (*s_ptr).P11_LED_colors
		;
		(*(*rt_ptr).data_ptr)[i,*,*] = readfits(file_list[i],header, /silent)
		;
		itime = 0L
		reads, FILE_BASENAME(file_list[i]), format='(4x,i8)',itime
		(*rt_ptr).exptime[i] = double(itime) / 1.0d6
		;
		(*s_ptr).P11_LED_colors[this_LED] = 'green'
		WCT_LED_stack, (*s_ptr).P11_LED_wid[1], (*s_ptr).P11_LED_colors
	endfor
	;
	MSP_BDP_print_text, Event, "Read in "+STRING(format='(i4.1)',rt_count)+' images'
	;
	P11_button_sensitize, Event, 4, 1
    ;
    P11RLCI_end:
	;
end
;
;------------------------------------------------------------
pro P11_Compute_Read_Time_Image, Event
	;
	MSP_BDP_print_text, Event, "Compute Read Time Image requested
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	rt_ptr = PTR_struct.PTR_RTime
	n_ptr = PTR_struct.PTR_night
	;
	deleted_data_value = -1.0e6
	;
	;	number of images
	;
	cube_size = SIZE( (*(*rt_ptr).data_ptr) )
	n_images = cube_size[1]
	nc = cube_size[2]
	nr = cube_size[3]
	;
	;print, "N_images, NC, NR = ",n_images, nc, nr
	;
	;	create an empty image to hold the times
	;
	out_image = fltarr(nc,nr)
	;
	;	and a header from the input list
	;
	header = headfits( (*rt_ptr).path_0, /silent)
	;
	n_LEDs = N_ELEMENTS( (*s_ptr).P11_LED_colors )
	LED_scale = float(nc) / float(n_LEDs)
	if(LED_scale le 1.0) then LED_scale = 1.0
	(*s_ptr).P11_LED_colors[*] = 'yellow'
	if(nc lt n_LEDs) then (*s_ptr).P11_LED_colors[nc:n_LEDs-1] = 'black'
	WCT_LED_stack, (*s_ptr).P11_LED_wid[2], (*s_ptr).P11_LED_colors
	;
	;	start the run through the pixels
	;
	the_times = (*rt_ptr).exptime[0:n_images-1]
	;
	for ic = 0, nc-1 do begin
		;
		this_LED = ROUND (float(ic) / LED_scale)
		this_LED = 0 > ((n_LEDs - 1) < this_LED)
		;
		(*s_ptr).P11_LED_colors[this_LED] = 'orange'
		WCT_LED_stack, (*s_ptr).P11_LED_wid[2], (*s_ptr).P11_LED_colors
		;
		for ir = 0, nr-1 do begin
			;
			;	capture the vector data from the array
			;
			vec = REFORM( (*(*rt_ptr).data_ptr)[*,ic,ir] )
			time = the_times
			;
			;	remove deleted data
			;
			ind = WHERE(vec[*] ne deleted_data_value, count)
			if(count gt 2) then begin
				vec = vec[ind]
				time = time[ind]
				;
				;	now, if Linearity correction worked, counts will be linear with exposure time
				;
				;	do linear regression of counts vs time
				;
				c = LINFIT(time, vec)
				;
				read_time = (c[0]/c[1])
				;
				out_image[ic,ir] = read_time
			endif else begin
				out_image[ic,ir] = deleted_data_value
			endelse
		endfor
		;
		(*s_ptr).P11_LED_colors[this_LED] = 'green'
		WCT_LED_stack, (*s_ptr).P11_LED_wid[2], (*s_ptr).P11_LED_colors
	endfor
	;
	;	output directory - test and create if necessary
	;
	out_dir = (*s_ptr).root_reduced + '11_Readout_Time' + PATH_SEP()
	test = FILE_INFO(out_dir)
	if(NOT(test.exists) or NOT(test.directory)) then FILE_MKDIR, out_dir
	;
	;	create output path
	;
	(*rt_ptr).outpath = out_dir + 'readout_time_image.fits'
	;
	;	update header
	;
	FXADDPAR, header, 'FILENAME', FILE_BASENAME( (*rt_ptr).outpath), 'MSP_BDP Created'
	FXADDPAR, header, 'OBJECT', 'Readout Time [s]'
	FXADDPAR, header, 'HISTORY', 'Computed Readout time on '+SYSTIME()
	;
	writefits, (*rt_ptr).outpath, out_image, header, /silent
	;
	MSP_BDP_print_text, Event, "Wrote image "+(*rt_ptr).outpath+" to disk"
	;
	;	free the cube pointer
	;
	PTR_FREE, (*rt_ptr).data_ptr
	;
	P11_button_sensitize, Event, 5, 1
	;
end
;
;----------------------------------------------------------------------
pro P11_Analyze_Read_Time_Image, Event
	;
	MSP_BDP_print_text, Event, "Analyze Read Time Image requested
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	rt_ptr = PTR_struct.PTR_RTime
	n_ptr = PTR_struct.PTR_night
	;
	deleted_data_value = -1.0e6
	;
	;	read the read time image and do a histogram analysis
	;
	image = readfits((*rt_ptr).outpath, header, /silent)
	;
	image_size = SIZE(image)
	nc = image_size[1]
	nr = image_size[2]
	;
	;	if full image, then skip top and bottom ski jump area
	;
	if(nc eq 1024 and nr eq 1026) then begin
		nc0 = 10
		nc1 = 1020
		nr0 = 10
		nr1 = 1020
		dc = 10
		dr = 10
	endif else begin
		nc0 = 0
		nc1 = nc
		nr0 = 0
		nr1 = nr
		dc = 5
		dr = 5
	endelse
	;
	sub_image = image[nc0:nc1:dc, nr0:nr1:dr]
	sub_image_size = SIZE(sub_image)
	ncs = sub_image_size[1]
	nrs = sub_image_size[2]
	npix = long(ncs) * long(nrs)
	;
	vec = REFORM(sub_image, npix)
	ind = WHERE(vec[*] ne deleted_data_value, count)
	if(count gt 4) then begin
		;
		zz = median_filtered_mean(vec)
		;
		MSP_BDP_print_text, Event, "Mean, sigma = "+STRING(format='(F12.5,3x,F12.5," s")',zz[0],zz[1])
		;
		(*rt_ptr).read_time_value = zz[0]
		WIDGET_CONTROL, (*s_ptr).P11_text_wid[0], SET_VALUE=STRING(format='(f8.6)',zz[0])
		;
	endif
end
;
;---------------------------------------------------------------------------
pro P11_end, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	a_ptr = PTR_struct.PTR_Ave_RAW1
	rt_ptr = PTR_struct.PTR_RTime
	;
	;	free the difference pointer
	;
	PTR_FREE, (*rt_ptr).data_ptr
	;
	;
	MSP_BDP_print_text, Event, "Flat Field Creation Completed"
	;
end