pro MSP_BDP_Read_BPM_Headers, Event, summary_struct
	;
	;	read the BPM directory to get the image list,
	;	then read each header to get the night and UT
	;	for each image into a summary structure
	;
	MSP_BDP_print_text, Event, "Fetch BPM List Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	f_ptr = PTR_struct.PTR_FITS
	b_ptr = PTR_struct.PTR_BPM
	;
	;	first, build the directory path to search
	;
	root_reduced = (*s_ptr).root_reduced
	BPM_directory = (*b_ptr).bpm_directory		; what if not yet loaded?
	if(BPM_directory eq '') then BPM_directory = '5_Bad_Pixel_Masks'
	;
	BPM_dir_path = root_reduced + BPM_directory + PATH_SEP()
	;
	;print, "BPM_path = ",BPM_dir_path
	;
	;	get list of images in that directory
	;
	test_dir = FILE_INFO(BPM_dir_path)
	if( NOT(test_dir.exists) or NOT(test_dir.directory) ) then begin
		MSP_BDP_print_text, Event, "ERROR - BPM directory path incorrect"
		goto, MBPBH_end
	endif
	;
	BPM_list = FILE_SEARCH(BPM_dir_path+'*.fits')
	;
	;print, "BPM_list = ",BPM_list
	;
	n_bpm = N_ELEMENTS(BPM_list)
	;
	MSP_BDP_print_text, Event, "Found "+STRING(format='(I)',n_bpm)+" BPM images."
	;
	;	open each and build the date, UT, path list
	;
	(*b_ptr).bpm_n_BPM = n_bpm
	(*b_ptr).bpm_mode = 1
	;
	for ibpm = 0, n_bpm-1 do begin
		bpm_path = BPM_list[ibpm]
		(*b_ptr).bpm_image_name[ibpm] = bpm_path
		;
		;	read the BPM image
		;
		header_BPM = headfits(bpm_path, /silent)
		;
		;	load buffers
		;
		n_header_lines = N_ELEMENTS(header_BPM)
		(*s_ptr).raw_header[0:n_header_lines-1] = header_BPM
		(*s_ptr).raw_header[n_header_lines:N_ELEMENTS((*s_ptr).raw_header)-1] = ''
		;
		;	extract key values
		;
		MSP_BDP_Parse_FITS_Header, Event
		;
		(*b_ptr).bpm_UT[ibpm] = (*f_ptr).ut
		raw_filename = (*f_ptr).raw_filename
		night_desig = 0L
		reads, raw_filename, format='(4x,i8)',night_desig
		(*b_ptr).bpm_night[ibpm] = night_desig
		;
		;	save JD value
		;
		bpm_desig = (*b_ptr).bpm_night[ibpm]
		year = bpm_desig / 10000L
		month = (bpm_desig - 10000L * year) / 100L
		day = bpm_desig - 10000L * year - 100L * month
		;
		bpm_UT = (*b_ptr).bpm_UT[ibpm]
		hour = FLOOR(bpm_UT)
		minute = FLOOR((bpm_UT - double(hour))*60.0d0)
		second = 3600.0d0 * (bpm_UT - double(hour) - double(minute)/60.0d0)
		;
		bpm_jd = JULDAY(month, day, year, hour, minute, ROUND(second))
		;
		(*b_ptr).bpm_jd[ibpm] = bpm_jd
		;
		;print, "BPM No. ",ibpm," Night, UT = ",night_desig, (*b_ptr).bpm_UT[ibpm], " path = ",bpm_path, $
		;		" JD = ",bpm_jd
	endfor
	;
	MBPBH_end:
end