function P3_build_clocking_error_mask, result, mask_value=mask_value, good_value=good_value

;
;  Accepts the input of detect_clocking_errors.pro and generates a Mimir image mask with a user supplied value
;  at the bad pixel locations and a different user supplied value everywhere else.  If no values are supplied,
;  0 is used as the mask_value and 1 as the good_value
;
;  MP  20090731  Original Version
;  MP  20100302  Removed double column masking in top half of detector since resolving bad column ambiguity.
;                Corrected intermittant column indexing error when last column of eight is bad.

if n_elements(mask_value) eq 0 then mask_value = 0
if n_elements(good_value) eq 0 then good_value = 1

;result = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
;[UL, ULcol, ULcol, UR, URcol, URcol, LL, LLcol, LLcol, LR, LRcol, LRcol]

mask = intarr(1024, 1026, /nozero) + good_value

;check upper left quadrant and mask if needed
if result[0] eq 1 then begin; quadrant needs masking
  for zone=0, 63 do begin
    for row=513, 1025 do begin
      ;mask[zone*8+result[1],row] = mask_value
      mask[zone*8+result[2] < 1023,row] = mask_value
    endfor
  endfor
endif

;check upper right quadrant and mask if needed
if result[3] eq 1 then begin; quadrant needs masking
  for zone=0, 63 do begin
    for row=513, 1025 do begin
      mask[512 + zone*8+result[4] < 1023,row] = mask_value
      ;mask[512 + zone*8+result[5],row] = mask_value
    endfor
  endfor
endif

;check lower left quadrant and mask if needed
if result[6] eq 1 then begin; quadrant needs masking
  for zone=0, 63 do begin
    for row=0, 512 do begin
      ;mask[zone*8+result[7],row] = mask_value
      mask[zone*8+result[8],row < 511] = mask_value; this is the bad column in the lower left quad
    endfor
  endfor
endif

;check lower right quadrant and mask if needed
if result[9] eq 1 then begin; quadrant needs masking
  for zone=0, 63 do begin
    for row=0, 512 do begin
      mask[512 + zone*8+result[10],row < 511] = mask_value; bad column in LR quad
      ;mask[512 + zone*8+result[11],row] = mask_value
    endfor
  endfor
endif

return, mask
end