pro P5_Make_Bad_Pixel_Mask, Event, input_pathname_list, output_pathname, colors, this_LED
	;
	;   processes numerous Mimir files to generate a bad pixel mask image
	;   for upper or lower array halves or the entire array
	;
	;   DPC  8/28/2004
	;        2004/10/27 - updated to be able to use any size images
	;                     will query user for sizes different than read in,
	;                     for example to just process first read, or UL UR
	;     	20050608 - modified to allow use of file of filenames
	;     	20050613 - modified to call ski_jump_fit_sub to remove ski_jump trend before analysis
	;     	20050902 - modified to include command line arguments to allow calling by batch.pro
	;     	20060626 - fixed bug preventing running with batch.pro
	;     	20060725 - trapped for reversed cold, hot cut values and excess bad pixels
	;		20071204	copied from make_bad_pix_mask.pro
	;
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	b_ptr = PTR_struct.PTR_BPM
	;
	high_cut = (*b_ptr).bpm_high_cut
	low_cut = (*b_ptr).bpm_low_cut
	;
	xh = high_cut
	xc = low_cut
	;
	;   TEST for high_cut > low_cut (KEJ test)
	;
	if(xc gt xh) then begin
	    ;print, "ERROR - low cut value HIGHER than high cut value, swapping values and continuing..."
	    xcs = xc
	    xc = xh
	    xh = xcs
	endif
	;
	image = long(readfits(input_pathname_list[0],header,/SILENT))
	image_size = size(image)
	;
	colors[this_LED] = 'yellow'
	WCT_LED_stack, (*s_ptr).P5_LED_wid[0], colors
	;
	;   now, generate histogram of data values
	;
	ymin = -200
	ymax = 200
	nb = 256 < (ymax - ymin)
	;
	;   invoke ski_jump correction (20050613)
	;
	image = bdp_ski_jump_fit_sub(image)
	;
	colors[this_LED] = 'orange'
	WCT_LED_stack, (*s_ptr).P5_LED_wid[0], colors
	;
	col0 = 2
	col1 = 1020
	row0 = 2
	row1 = 1023
	;
	nvec = long(row1-row0+1) * long(col1-col0+1)
	;
	vec = REFORM(image[col0:col1,row0:row1],nvec)
	;
	help, vec
	;
	h = dpc_histogram(vec, MAX=ymax, MIN=ymin, NBINS=nb)
	x = ymin + (ymax - ymin) / (nb -1.) * FIndGen(nb)
	dummy = where(vec lt ymin or vec gt ymax, nbelow)
	;
	colors[this_LED] = 'blue'
	WCT_LED_stack, (*s_ptr).P5_LED_wid[0], colors
	;
	;print, "Number of pixels below YMIN or above YMAX = ",nbelow
	;
	;   for all the rest of the files, do same reading, histogram accumulation, finally plotting
	;   result
	;
	nfiles = N_ELEMENTS(input_pathname_list)
	;
	nfiles = 2	;(for now)
	;
	if (nfiles ne 0) then begin
	    for nf = 1, nfiles-1 do begin       ; read the rest of the image files
	       image_n = long(readfits(input_pathname_list[nf], header, /SILENT))
	       ;
	       colors[this_LED] = 'yellow'
		   WCT_LED_stack, (*s_ptr).P5_LED_wid[0], colors
	       ;
	       ;    invoke ski_jump correction (20050613)
	       ;
	       image_n = bdp_ski_jump_fit_sub(image_n)
	       ;
	       colors[this_LED] = 'orange'
		   WCT_LED_stack, (*s_ptr).P5_LED_wid[0], colors
	       ;
	       vec = image_n[col0:col1,row0:row1]
	      ; hnew = HISTOGRAM( image[col0:col1,row0:row1] , NBINS=nb, OMAX = hmax, OMIN = hmin, MAX=ymax, MIN=ymin, INPUT = h)
	       hnew = dpc_histogram(vec, MAX=ymax, MIN=ymin, NBINS=nb)
	       dummy = where(vec lt ymin or vec gt ymax,nbelow_i)
	       h = h + hnew
	       nbelow = nbelow + nbelow_i
	       ;
	       colors[this_LED] = 'blue'
		   WCT_LED_stack, (*s_ptr).P5_LED_wid[0], colors
		   ;
	    endfor
	endif
	;
	;print, "Processed ",nfiles," Mimir Files"
	;
	;	plot into widget draw window
	;
	device, decomposed=0
	window_id = (*s_ptr).P5_draw_window_id
	;
	WSET, window_id
	;
	;
	plot, x, h, /YLOG,yrange=[1,max(h)],xrange=[ymin,ymax],xstyle=1,psym=10
	;
	xs=50.
	ys=3000.
	;
	restart:    MSP_BDP_print_text, Event, "Select Low Cut Limit"
;	    ;
;	    cursor, xc, yc, /UP, /DATA
;	    ;
;	    ;   draw separating line, count histogram elements leftward, report normalized sum
;	    ;
;	    print, "X, Y position of cursor = ",xc,yc
;	    ;
;	    ;plot, x, h, /YLOG,yrange=[1,max(h)],xrange=[-100.,65536.],xstyle=1,psym=10
;	    xd = [xc,xc]
;	    yd = [1.,6000.]
;	    PlotS, xd, yd, linestyle=2, thick=3
;	    ;
;	    cold = string(format="(F6.0)",xc)
;	    XYoutS, xc, 20000., cold, alignment=0.5
;	    print,"Cold Pixel Count Limit = ",xc
;	    ;
;	    ;   compute max bin number
;	    ;
;	    maxbin = long(((xc - ymin) / (ymax - ymin))*nb + 1)
;	    ;
;	    print, "maxbin = ",maxbin
;	    ;
;	    nbad_cold = total(long(h[0:maxbin]))
;	    n_normal_cold = float(nbad_cold) / nfiles[1]
;	    ;
;	    print, "Number of cold bad pixels per image = ",n_normal_cold
;	    ;
;	    xs = 50.
;	    ys = 300.
;	    ;
;	    MSP_BDP_print_text, Event, "Select Hot Pixel Cut Limit"
;	    ;
;	    cursor, xh, yh, /UP, /DATA
;	    xd = [xh,xh]
;	    yd = [1.,600.]
;	    PlotS, xd, yd, linestyle=2, thick=3
;	    hot = string(format="(f6.0)",xh)
;	    XYoutS, xh, 2000., hot, alignment=0.5
;	    print,"Hot Pixel Count Limit = ",xh
;	    ;
;	    ;   test that hot is higher than cold
;	    ;
;	    if(xc gt xh) then begin
;	        print, "ERROR - Cold cut is hotter than hot cut. Reversing values and continuing..."
;	        xcs = xc
;	        xc = xh
;	        xh = xcs
;	    endif
;	    ;
;	    ;   compute min bin number
;	    ;
;	    minbin = long(nb-1 - ((ymax - xh) / (ymax - ymin))*nb)
;	    ;
;	    print, "minbin = ",minbin
;	    ;
;	    nbad_hot = total(long(h[minbin:nb-1]))
;	    n_normal_hot = float(nbad_hot) / nfiles[1]
;	    ;
;	    print, "Number of hot bad pixels per image = ",n_normal_hot
;	    ;
;	    print, "Number of pixels outside histogram range per image = ",nbelow / nfiles[1]
;	    ;
;	    print, "Total number of bad pixels per image = ", n_normal_cold + n_normal_hot + (nbelow / nfiles[1])
;	    ;
;	    print, "Fraction of good pixels per image = ",100.*(delta_col*delta_row - n_normal_cold - n_normal_hot)/(delta_col*delta_row)
;	    ;
;	    ;
;	;
;	;wdelete, 12
;	;cursor, x, y, /DOWN
;	;
;	;   create bad pixel mask, using these hot, cold values as limits
;	;
;	;   start with a clean mask, then using each input image, accumulate bad
;	;   pixel counts. After that, save as bad only those pixels whose count
;	;   is > 75% of the number of input files.
;	;
;	bad_count_mask = IntArr(delta_col,delta_row+2)
;	;
;	ixc = long(xc)
;	ixh = long(xh)
;	for ifiles = 0, nfiles-1 do begin
;	    image = readfits(input_pathname_list[ifiles],header)
;	    ;
;	    ;   invoke ski_jump_correction (20050613)
;	    ;
;	    image = ski_jump_fit_sub(image)
;	    ;
;	    for irow = row0, row1+2 do begin
;	        jcold = where((image[*,irow] lt ixc) or (image[*,irow] gt ixh), nrowbad)  ; find all bad pixels in a row
;	        if (nrowbad gt 0) then bad_count_mask[jcold, irow-row0] = bad_count_mask[jcold, irow-row0] + 1
;	    endfor
;	endfor
;	;
;	frac_test = long(0.75*nfiles[1])
;	;
;	bad_mask = IntArr(delta_col,delta_row+2)
;	;
;	bcount = 0L
;	for irow = row0, row1+2 do begin
;	    jfrac = where(bad_count_mask[*,irow-row0] ge frac_test, nrowgood)
;	    if (nrowgood gt 0) then begin
;	       bad_mask[jfrac,irow-row0] = 1
;	       bbcount = size(jfrac)
;	       bcount = bcount + bbcount[1]
;	    endif
;	endfor
;	;
;	print, "Total number of bad pixels in Bad Pixel Mask = ",bcount
;	;
;	;   test for really high bad pixel counts
;	;
;	if(bcount gt 10000) then begin
;	    print,"ERROR!!! Too many bad pixels -- something is wrong. STOP and fix the problem."
;	    stop
;	endif
;	;
;	;writefits, output_pathname, bad_mask, header
;	;
;	print,"prepared to write output image ", output_pathname
;
end


