function better_ramp, x, y, threshold, npts, order
;
;	fits ramp data, excluding one point at a time,
;	for nearest npts points, with fit to order order,
;	and detection at threshold sigma
;
n = n_elements(x)
y_replace = y * 0.0
for i = 0, n-1 do begin
	;
	this_x = x[i]
	;
	dif = abs(this_x - x)
	ind = SORT(dif)
	;
	;	skip first element, as self
	;
	ind_npts = ind[1:npts]
	;
	test_x = x[ind_npts]
	test_y = y[ind_npts]
	;
	;	fit and rms
	;
	result = POLY_FIT(test_x,test_y,order,YFIT=yfit)
	resid = test_y - yfit
	zz = median_filtered_mean(resid)
	rms = zz[1]
	;
	;	project for this_x
	;
	this_y = 0.0
	for j = 0, order do begin
		this_y += result[j] * (this_x ^ j)
	endfor
	;
	deviation = abs(this_y - y[i])
	if(deviation/rms gt threshold) then	y_replace[i] = this_y else y_replace[i] = y[i]
	;
	a = 0
endfor
;
return, y_replace
;
end

pro clean_ramp, x, y
;
;	takes a linearity ramp (x, y), finds deviant pixels, and replaces
;	them with fitted values
;
;	DPC	20100709
;
;---------------------------------------------------------------
;
;	for every pixel, find nearest m pixels and do a parabolic fit
;
;
;	first pass, linear fit with 10pts, threshold = 25 sigma
;
y_new = better_ramp(x, y, 25., 10, 1)
;
;	then, parabolic, 5 pts
;
y = better_ramp(x, y_new, 15., 5, 2)
;
end

pro test_clean_ramp
;
;	reads a pixel ramp, plots it,
;	locates deviant pixels, removes them,
;	finds saturation level in remaining
;	ramp
;
;	DPC	20100709
;
file_in = dialog_pickfile(title="select ramp file")
openr, lu_ramp, file_in, /GET_LUN
;
x = dblarr(1)
y = dblarr(1)
n = 0L
;
WHILE ~EOF(lu_ramp) do begin
	readf, lu_ramp, xx, yy
	if(n eq 0L) then begin
		x[0] = xx
		y[0] = yy
	endif else begin
		x = [x, xx]
		y = [y, yy]
	endelse
	n++
ENDWHILE
;
free_lun, lu_ramp
x_plot = x
y_plot = y
;
window, 11
x_text = 'x'
y_text = 'y'
plot, x_plot, y_plot,yrange=[0.0,max(y_plot)],xrange=[-1.0,max(x_plot)],xstyle=1,psym=-4,$
;				color=1, background=0, $
				thick=1, LINESTYLE=0, Ytitle="CDS ADUs", $
				Xtitle = "Exp. Time", TITLE="Pixel  ("+x_text+','+y_text+')', $
				;POSITION=(*b_ptr).position,/DEVICE,
				xtickinterval=1.0, $
				xminor=4
;
y_in = y
clean_ramp, x, y
;
;	plot deviations
;
ind_bad = WHERE(y_in ne y, nbad)
if(nbad gt 0) then begin
	print, "found ",nbad," bad values"
	oplot, x[ind_bad], y[ind_bad], psym=6, thick=3	;, color=1
endif
;
end