pro coadd_median_with_bad_sub, Event, bpf, pathlist, outpath, $
								row513_flag, LED_wid, this_LED, colors, $
								MBF = mbf
	;
	;   adds (averages) Mimir images, from list of file pathnames supplied
	;   using a median filter for each pixel, rejecting
	;   outliers, if possible
	;
	;   normally called by Coadd_Linearity_Images.pro
	;
	;   DPC 20050611 modified from coadd_median_with_bad.pro to work as a subroutine
	;       20051012 fixed file echo bug (thanks Frannie!)
	;       20051123 modified to include relative scaling by image means (median filtered), and
	;                    added pixel deletion conditions
	;       20051210 modified to trap incoming and outgoing pixels with values *below* bad_pix_value
	;		20091127 added model background removal, if flag set
	;		20101227 added restore of night's data structure (how did this work before??)
	;		20121120 removed restore of night's data, as the pointers are already called out
	;
	;-----------------------------------------
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	n_ptr = PTR_struct.PTR_night
	;
	if(N_ELEMENTS(mbf) eq 0) then mbf = 0; if no flag present, don't model background
										 ; if flag present, use its value to tell
	;
	image = readfits(pathlist[0],header,/silent)
	i_size = SIZE(image)
	ncol = i_size[1]
	nrow = i_size[2]
	nimage = N_ELEMENTS(pathlist)
	;
	;	restore night summary structure if not already present
	;
	image_file = FILE_BASENAME(pathlist[0])
	parts = STRSPLIT(image_file, '.', COUNT=count, /EXTRACT)
	if(count gt 0) then begin
		night_desig = long(parts[0])
		;
		;	compare to current night designation
		;
		if(night_desig ne (*n_ptr).night_desig) then begin
			MSP_BDP_Create_Night_File_Path, Event, night_desig, night_file_path
			MSP_BDP_Restore_Night_File, Event, night_file_path, description, ierr
			WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
			s_ptr = PTR_struct.PTR_summary
			l_ptr = PTR_struct.PTR_Linearity
			n_ptr = PTR_struct.PTR_night
		endif
	endif
	;
	bad_pix_value = -1.0e6
	;
	;   load the bad pixel mask
	;
	bad_image = readfits(bpf,header,/silent)
	;
	;   read in each file
	;
	image_data_ptr = PTR_NEW(fltarr(nimage,ncol,nrow))     ; do up to nfiles-1 images
	accum_ptr = PTR_NEW(fltarr(ncol,nrow))
	;
	smooth_ptr = PTR_NEW(fltarr(ncol, nrow))				; the image after background modeling and removal
	;
	ngood = 0
	path = ''
	for ifile = 0L, nimage-1 do begin
	    (*image_data_ptr)[ifile,*,*] = float(readfits(pathlist[ifile],header,/silent))
	    ;
	    ;	check the filter designation of the group by looking at the
	    ;	first image header and what is stored for the group
	    ;
	    ;	but, skip this test for linearity file building. (filenames with "temp_" in them)
	    ;
	    if(ifile eq 0) then begin
	    	;
	    	save_header = header
	    	;
	    	;	check filter list in this header against group table's list
	    	;	in case we had to edit them in the group table
	    	;
	    	;		first, parse the pathname to get image ID, if none, jump around
	    	;
	    	image_file = FILE_BASENAME(pathlist[ifile])		; get rid of directory stuff
	    	;
	    	;	check against "temp_" in the name
	    	;
	    	t_test = STRPOS(image_file, 'temp_')
	    	;
	    	if(t_test eq 0) then goto, loop_around
	    	;
	    	parts = STRSPLIT(image_file, '.', COUNT=count, /EXTRACT)
	    	if(count lt 2) then goto, loop_around
	    	;
	    	;	and test for whether the image number was tagged with higher level info ("_LC" e.g.)
	    	sub_parts = STRSPLIT(parts[1], '_', COUNT=subcount, /EXTRACT)
	    	;
	    	if(subcount gt 0) then parts[1] = sub_parts[0]
	    	;
	    	image_no = long(STRTRIM(parts[1],2))
	    	;
	    	;print, "in coadd - image number from pathname = ",image_no
	    	;
	    	;	now, match image_no to group membership
	    	;
	    	filter_name = 'none'
	    	n_groups = (*n_ptr).num_groups
	    	for ig = 0, n_groups-1 do begin
	    		;
	    		;	look at each group list for a matching image_number entry
	    		;
	    		n_g_images = (*n_ptr).groups[ig].group_images
	    		;
	    		;print, "   in group ",ig," number of images = ",n_g_images
	    		;
	    		ind = WHERE(image_no eq (*n_ptr).groups[ig].group_list[0:n_g_images-1], icount)
	    		;
	    		;print, "   for this group, matched to ",icount," images on its list"
	    		;
	    		if(icount ne 0) then begin
	    			;
	    			;	matched. get group filter
	    			;
	    			filter_name = (*n_ptr).groups[ig].group_filter
	    			;
	    			;print, "(coadd_median_with_bad_sub) Matched! filter_name = ",filter_name
	    			;
	    			goto, loop_out
	    		endif
	    	endfor
	    	;
	    	;	if we didn't match, write error and jump around
	    	;
	    	if(filter_name eq 'none') then begin
	    		MSP_BDP_print_text, event, "Error (coadd_median_with_bad_sub) - failed to update filter name from group table"
				print, "Image_no = ",image_no,"  Subcount = ",subcount
				stop
	    		goto, loop_around
	    	endif
	    	;
	    	loop_out:			; OK, got a good group filter name, check this header
	    	;
	    	Test_Fix_Bad_Filter_Names, Event, filter_name, HEADER=save_header
	    	;
	    	;			or not
	    	;
	    	loop_around:
	    	;
	    endif
	endfor
	;
	colors[this_LED] = 'orange'
	WCT_LED_stack, LED_wid, colors
	;
	MSP_BDP_print_text, Event, "Read "+STRING(format='(i)',nimage)+" input images"
	;
	;   do median filtered mean calculation with bad pixel masking
	;
	;   first, get means for each image
	;
	image_mean = dblarr(nimage)
	;
	for ifile = 0, nimage-1 do begin
		;
		;	subsample by 10x10 to speed things along
		;
	    image_single = REFORM( (*image_data_ptr)[ifile,0:*:10,0:*:10])
	    im_size = SIZE(image_single)
	    nc = im_size[1]
	    nr = im_size[2]
	    image_single_vec = REFORM(image_single, long(nc)*long(nr))
	    image_vec = image_single_vec[WHERE(image_single_vec gt -10000. and image_single_vec lt 100000.,icount)]
	    if(icount ge 100) then begin
	    	zz = median_filtered_mean(image_vec)
	    endif else begin
	    	zz = [0.,100.]
	    endelse
	    image_mean[ifile] = zz[0]
	endfor
	;
	colors[this_LED] = 'red'
	WCT_LED_stack, LED_wid, colors
	;
	;   compute relative scale factors -- we effectively scale to means before combining
	;
	zz = median_filtered_mean(image_mean)
	set_mean = zz[0]
	scale_factor = dblarr(nimage)
	scale_factor = image_mean / set_mean
	;
	ind500 = WHERE(image_mean gt 500, count_500)
	nimage_half = ROUND(nimage/2.0)
	;
	;   the pixel evaluation and combining engine
	;
	icolor = 'blue'
	for icol = 0L, ncol-1 do begin
		;
		icol25 = round(icol/25L)*25L
		if(icol25 eq icol) then begin
			colors[this_LED] = icolor
			WCT_LED_stack, LED_wid, colors
			if(icolor eq 'blue') then icolor = 'violet' else icolor = 'blue'
		endif
		;
	    for irow = 0, nrow-1 do begin
	       if(bad_image[icol,irow] ne 1) then begin
	       		pix_vec = REFORM( (*image_data_ptr)[*,icol,irow] )
	       		;
	       		ind_good = WHERE(pix_vec ne bad_pix_value, count_good)
	       		if(count_good ge nimage_half) then begin
	       			pix_vec = pix_vec[ind_good]	* scale_factor[ind_good]	; strip deleted pixels, scale others
	       			;
					if(count_500 gt 0) then begin
						pix_frac = pix_vec / image_mean[ind_good]
						ind_high = WHERE(pix_frac gt 0.1, count_high)
						if(count_high gt 0) then begin
							pix_vec = pix_vec[ind_high]
							;
							;	passed all but row513 test, so go ahead and calc
							;
			                mx = median_filtered_mean(pix_vec)
		                    ;
		                    (*accum_ptr)[icol,irow] = mx[0]     ; the final mean value
		                    ;
                 		endif else (*accum_ptr)[icol,irow] = bad_pix_value
			        endif else begin
			        	mx = median_filtered_mean(pix_vec[ind_good])
	                    ;
	                    (*accum_ptr)[icol,irow] = mx[0]     ; the final mean value
	                endelse
	             endif else (*accum_ptr)[icol,irow] = bad_pix_value
	       endif else (*accum_ptr)[icol,irow] = bad_pix_value
	    endfor
	endfor
	;
	;	test for row513 flag
	;
	if(row513_flag) then (*accum_ptr)[*,513] = bad_pix_value
	;
	;   all pixels processed, save
	;
	;   trap bad pixels below bad_pix_value
	;
	ind = WHERE( (*accum_ptr) lt bad_pix_value,count)
	if(count gt 0) then (*accum_ptr)[ind] = bad_pix_value
	;
	;	update header
	;
	FXADDPAR, save_header, "FILENAME", FILE_BASENAME(outpath), "Coadded file name"
	FXADDPAR, save_header, "DIRNAME", FILE_DIRNAME(outpath),"Directory Path for file"
	FXADDPAR, save_header, "HISTORY", "  Coadding on/at "+SYSTIME()
	FXADDPAR, save_header, "HISTORY","    reduced by "+(*s_ptr).who
	;
	if(size(bpf,/dimensions) ne 0) then bpf = bpf[0]
	;
	FXADDPAR, save_header, "HISTORY","   Using Bad Pix Mask "+FILE_BASENAME(bpf)
	FXADDPAR, save_header, "HISTORY", "  Coadded (averaged) file, composed of the following images:"
	for ifile = 0, nimage-1 do begin
		FXADDPAR, save_header, "HISTORY", STRING(format='(10x,a)',FILE_BASENAME(pathlist[ifile]))
	endfor
	;
	if(mbf eq 0) then begin
		writefits,outpath,(*accum_ptr),save_header
	endif else begin
		(*smooth_ptr) = bdp_model_background( (*accum_ptr), SKY=sky)
		FXADDPAR, save_header, "HISTORY", " Removed model background (sky ave = "+STRING(sky)
		FXADDPAR, save_header, "HISTORY", "   This is done to flatten the output image"
		writefits, outpath, (*smooth_ptr), save_header
	endelse
	;
	MSP_BDP_print_text,Event, "Wrote Coadded Image "+outpath+" to disk"
	;
	;	free pointers
	;
	PTR_FREE, accum_ptr, image_data_ptr, smooth_ptr
end