function linearize_old, input_image, bad_pix_image, lin_data, norder
;
;   linearizer engine
;
;   assumes input "image" is of form "Reset - RAW1" or "Reset - RAW2"
;   and so applicable by linearity files
;
;   DPC 20050624 initial code
;       20050714 modified to utilize order of LC corrections
;       20050715 modified to set zero saturation pixels to bad pix value
;       20050924 modified conditions for setting bad pixel value
;       20050925 continued tuning of rejection parameters
;       20060129 modified to work with subframed images
;
image_size = SIZE(input_image)
;
nrow = image_size[2]
ncol = image_size[1]
;
lin_size = SIZE(REFORM(lin_data[0,*,*]))
lin_col = lin_size[1]
lin_row = lin_size[2]
;
;   if the badpixel mask size and the linearity image have
;   different sizes, must compute offset row, col for extraction
;
if(lin_col ne ncol or lin_row ne nrow) then begin
    ;
    ;  subframe images are (mostly) symmetric about center
    ;
    col_offset = (lin_col/2) - (ncol/2)
    row_offset = (lin_row-2)/2 - (nrow-2)/2       ; the extra 2 is the top two rows
endif else begin
    col_offset = 0
    row_offset = 0
endelse
;
output_image = fltarr(ncol, nrow)
;
bad_value = -1.0e6
bad_band = abs(bad_value)/2.
sat_value = -1.0e6
;
;   map LC images to correction terms for different order numbers
;
case norder of
    2: begin
         cc2 = REFORM(lin_data[1,*,*])
         t0 = REFORM(lin_data[2,*,*])
         t1 = REFORM(lin_data[3,*,*])
         t2 = REFORM(lin_data[4,*,*])
       end
    3: begin
         cc2 = REFORM(lin_data[1,*,*])
         cc3 = REFORM(lin_data[2,*,*])
         t0 = REFORM(lin_data[3,*,*])
         t1 = REFORM(lin_data[4,*,*])
         t2 = REFORM(lin_data[5,*,*])
         t3 = REFORM(lin_data[6,*,*])
       end
    4: begin
         cc2 = REFORM(lin_data[1,*,*])
         cc3 = REFORM(lin_data[2,*,*])
         cc4 = REFORM(lin_data[3,*,*])
         t0 = REFORM(lin_data[4,*,*])
         t1 = REFORM(lin_data[5,*,*])
         t2 = REFORM(lin_data[6,*,*])
         t3 = REFORM(lin_data[7,*,*])
         t4 = REFORM(lin_data[8,*,*])
       end
endcase
;
for icol = 0, ncol-1 do begin
    icx = icol + col_offset
    nbad_in_col = 0
    for irow = 0, nrow-1 do begin
        iry = irow + row_offset
        ;
        ;  check bad pixel mask
        ;
        bad_test_1 = abs(input_image[icol,irow] - bad_value)/bad_band
        bad_test_2 = abs(input_image[icol,irow] + bad_value)/bad_band
        ;
        ; debug
        ;
        ;if(icol eq 1023 and irow eq 724) then print,"bad_test_1, _2 = ",bad_test_1,", ",bad_test_2
        ;
        if(bad_pix_image[icol, irow] ne 1 and bad_test_1 gt 1.0 and bad_test_2 gt 1.0) then begin       ; all bad pixel tests passed
            ;
            ;  test for saturation
            ;
            pix_value = double(input_image[icol, irow])
            ;
            ; debug
            ;
            ;if(icol eq 1023 and irow eq 724) then print,"pix_value, sat_value, cc2 value = ",pix_value, lin_data[0,icol,irow], cc2[icol,irow]
            ;
            if(pix_value le lin_data[0,icx,iry] and lin_data[0,icx,iry] ne 0 and (cc2[icx,iry] ne 0 or norder ne 2) ) then begin
                ;
                ;  OK, not bad, not saturated. Compute "time" value
                ;
                case norder of
                    2: begin
                          cc_2 = cc2[icx,iry]
                          t_0 = t0[icx,iry]
                          t_1 = t1[icx,iry]
                          t_2 = t2[icx,iry]
                          ;
                          t = t_0 + t_1*pix_value + t_2*pix_value*pix_value
                          corr = -1.0d0 * t * t * cc_2
                        end
                    3: begin
                          cc_2 = cc2[icx,iry]
                          cc_3 = cc3[icx,iry]
                          t_0 = t0[icx,iry]
                          t_1 = t1[icx,iry]
                          t_2 = t2[icx,iry]
                           t_3 = t3[icx,iry]
                          ;
                          t = t_0 + t_1*pix_value + t_2*pix_value*pix_value + t_3*(pix_value^3)
                          corr = -1.0d0 * (t * t * cc_2 + (t^3)*cc_3)
                        end
                    4: begin
                          cc_2 = cc2[icx,iry]
                          cc_3 = cc3[icx,iry]
                          cc_4 = cc4[icx,iry]
                          t_0 = t0[icx,iry]
                          t_1 = t1[icx,iry]
                          t_2 = t2[icx,iry]
                          t_3 = t3[icx,iry]
                          t_4 = t4[icx,iry]
                          ;
                          ;  debug
                          ;
                          ;if(icol eq 1023 and irow eq 724) then print, "cc3, cc4, t0, t1, t2, t3, t4 = ",cc_3, cc_4, t_0, t_1, t_2, t_3, t_4
                          ;
                          ;
                          t = t_0 + t_1*pix_value + t_2*pix_value*pix_value + t_3*(pix_value^3) + t_4*(pix_value^4)
                          corr = -1.0d0 * (t * t * cc_2 + (t^3)*cc_3 + (t^4) * cc_4)
                            ;
                            ; debug
                            ;
                            ;if(icol eq 1023 and irow eq 724) then print, "t, corr = ",t,corr
                            ;
                    end
                endcase
                ;
                ;  the correction should never be negative, but noise and bias fluctuations may make that happen
                ;   try to trap and zero this correction
                ;
                if (corr lt 0.) then begin
                    ;
                 ;   if correction large and negative, report this and set to zero
                 ;
                    if(corr lt -20.) then begin
                        ;print, "ERROR in Linearize - Correction negative for Pixel (X,Y) = ",icol,", ",irow
                        ;print, "                     Corr = ",corr,", T = ",t
                        ;print, "                     Setting Corr to 0"
                        corr = 0.
                    endif
                    ;
                    ;   if corr moderately negative, check pix value
                    ;
                    if(pix_value gt 0.) then begin
                        ;
                        ;  if pix_value positive just zero the correction
                        ;
                        corr = 0.
                    endif else begin
                        ;
                        ;  if pix_value negative...
                        ;
                        if(abs(pix_value) gt 300.) then begin
                            ;
                            ;  pixel value large and negative, set to deleted data value
                            ;
                            ;print, "ERROR in Linearize - Big Neg Pixel Value (",pix_value,") & Neg Correction at (X,Y)=",icol,",",irow
                            ;print, "                     Corr = ",corr," T = ",t
                            corr = -pix_value + bad_value
                        endif else begin
                            ;
                            ;  if value moderate, just set corr to zero
                            ;
                            corr = 0.
                        endelse
                    endelse
                endif
                ;
                ;
                ;  this is the "normal" place where a good correction is applied
                ;
                output_image[icol,irow] = pix_value + corr
                ;
            endif else begin
                ;
                ; to get here, either the pixel has a value exceeding its saturation value in the linearity tables
                ; or its correction terms are zeroed -- separate to see which
                ;
                ;if(lin_data[0,icol,irow] eq 0) then begin
                ;    print,"pixel at (C,R)= ",icol,",",irow," Value = ",input_image[icol,irow],$
                ;        " set to deleted data because saturation value = 0"
                ;endif else begin
                ;    if(input_image[icol,irow] gt lin_data[0,icol,irow]) then begin
                ;        print,"pixel at (C,R)= ",icol,",",irow," Value = ",input_image[icol,irow],$
                ;            " set to deleted data because it exceeded saturation (Value = ",lin_data[0,icol,irow],")"
                ;    endif else begin
                ;        print,"pixel at (C,R)= ",icol,",",irow," Value = ",input_image[icol,irow],$
                ;            " set to deleted data because CC2 correction is 0"
                ;    endelse
                ;
                ;endelse
                ;
                output_image[icol,irow] = sat_value
                ;
            endelse
       endif else begin
            ;
            ; pixel set to deleted data either because of bad pixel mask or because
            ;   input image came in with data inside the 50% band around +/- of the bad pixel value
            ;   -- this should rarely, if ever, occur ---
            ;   if it does, it probably signifies an error with accounting for coadding
            ;
            output_image[icol,irow] = bad_value
            ;
            ;if(bad_pix_image[icol,irow] ne 1) then print, "pixel at (C,R)= ",icol,",",irow," Value = ",input_image[icol,irow],$
            ;                         " set to deleted data because of bad pixel tests. BT1, BT2 = ",bad_test_1, bad_test_2
       endelse
       ;
       if(output_image[icol,irow] eq bad_value) then nbad_in_col++
    endfor
    ;
    if(nbad_in_col eq nrow) then begin
        print,"In Linearize, entire column bad. Col number = ",icol
    endif
endfor
;
return, output_image
;
end