function safe_math_new, image_a, image_b, bad_pix_image, operation
;
;   does simple math ops (+,-,/,*), but respects bad pixels properly
;
;   if second image is a constant, do constant arith
;
;   DPC 20050923    separated from linearity_correct.pro
;       20051123    modified to fix indexing bug
;       20051210    modified to trap pixels with values *below* bad_pix_value
;		20080117	sped up by simplifying traps and tests
;		20100114	sped up by removing WHERE statements and excessive tests
;		20100623	test for NaNs and replace with deleted
;
bad_pix_value = -1.0e6
;
nb = N_ELEMENTS(image_b)
if(nb eq 1) then iconst = 1 else iconst = 0
;
result = image_a
;
;	split constant second argument from image second argument to reduce
;	tests
;
good_mask_a = image_a + 0.5e6		; 1 where not deleted, 0 where
good_mask_a >= 0.0
good_mask_a <= 1.0
;
bad_mask_a = 1.0 - good_mask_a
bad_mask_a += bad_pix_image
bad_mask_a <= 1.0
;
if(iconst eq 0) then begin
	;
	;	second argument is an array
	;
	;	build masks of deleted, non-delted pixels for each array
	;
	good_mask_b = image_b + 0.5e6
	good_mask_b >= 0.0
	good_mask_b <= 1.0
	bad_mask_b = 1.0 - good_mask_b
	;
	;	if division, also flag as bad any image_b's with zeros
	;
	if(operation eq '/') then begin
		zero_mask_b = image_b eq 0.0
		bad_mask_b += zero_mask_b
		bad_mask_b <= 1.0
	endif
    ;
    ;  do the math
    ;
    case operation of
      '+':  result += image_b
      '-':  result -= image_b
      '/':  result /= (image_b * good_mask_b + bad_mask_b)	; turns 0s into 1s in B
      '*':  result *= image_b								; pixel by pixel mult
      else: begin
              print,"ERROR no proper operation selected"
              stop
		    end
    endcase
    ;
    ;	put deleted pixels back in
    ;
    result *= (good_mask_a AND good_mask_b)
    result += (bad_mask_a OR bad_mask_b) * bad_pix_value
    ;
endif else begin
	;
	;	the second argument is a constant
	;
	B = image_b
	if(B ne 0.0) then begin
	    ;
	    ;  do the math
	    ;
        case operation of
          '+':  result += B
          '-':  result -= B
          '/':  result /= B
          '*':  result *= B
          else: begin
		              print,"ERROR no proper operation selected"
		              stop
		        end
        endcase
        ;
        ;	put deleted back in
        ;
        result *= good_mask_a
    	result += (bad_mask_a * bad_pix_value)
	    ;
	endif else begin
		result = bad_pix_value
	endelse
    ;
endelse
;
ind_nan = WHERE(~FINITE(result), n_nan)
if(n_nan gt 0) then begin
	result[ind_nan] = bad_pix_value
endif
;
return, result
end